# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from pyghmi import exceptions as pyghmi_exception
from pyghmi.ipmi import command as ipmi_command

from os_faults.api import error
from os_faults.api import power_management
from os_faults import utils

LOG = logging.getLogger(__name__)


BMC_SCHEMA = {
    'type': 'object',
    'properties': {
        'address': {'type': 'string'},
        'username': {'type': 'string'},
        'password': {'type': 'string'},
    },
    'required': ['address', 'username', 'password']
}


class IPMIDriver(power_management.PowerDriver):
    """IPMI driver.

    **Example configuration:**

    .. code-block:: yaml

        power_managements:
        - driver: ipmi
          args:
            mac_to_bmc:
              aa:bb:cc:dd:ee:01:
                address: 170.0.10.50
                username: admin1
                password: Admin_123
              aa:bb:cc:dd:ee:02:
                address: 170.0.10.51
                username: admin2
                password: Admin_123
            fqdn_to_bmc:
              node3.local:
                address: 170.0.10.52
                username: admin1
                password: Admin_123

    parameters:

    - **mac_to_bmc** - list of dicts where keys are the node MACs and
      values are the corresponding BMC configurations with the folowing
      fields:

      - **address** - ip address of IPMI server
      - **username** - IPMI user
      - **password** - IPMI password
    """

    NAME = 'ipmi'
    DESCRIPTION = 'IPMI power management driver'
    CONFIG_SCHEMA = {
        'type': 'object',
        '$schema': 'http://json-schema.org/draft-04/schema#',
        'properties': {
            'mac_to_bmc': {
                'type': 'object',
                'patternProperties': {
                    utils.MACADDR_REGEXP: BMC_SCHEMA
                }
            },
            'fqdn_to_bmc': {
                'type': 'object',
                'patternProperties': {
                    utils.FQDN_REGEXP: BMC_SCHEMA
                }
            },
        },
        'anyOf': [
            {'required': ['mac_to_bmc']},
            {'required': ['fqdn_to_bmc']},
        ],
        'additionalProperties': False,
    }

    def __init__(self, params):
        self.mac_to_bmc = params.get('mac_to_bmc', {})
        self.fqdn_to_bmc = params.get('fqdn_to_bmc', {})
        # TODO(astudenov): make macs lowercased

    def _find_bmc_by_host(self, host):
        if host.mac in self.mac_to_bmc:
            return self.mac_to_bmc[host.mac]
        if host.fqdn in self.fqdn_to_bmc:
            return self.fqdn_to_bmc[host.fqdn]

        raise error.PowerManagementError(
            'BMC for {!r} not found!'.format(host))

    def _run_set_power_cmd(self, host, cmd, expected_state=None):
        bmc = self._find_bmc_by_host(host)
        try:
            ipmicmd = ipmi_command.Command(bmc=bmc['address'],
                                           userid=bmc['username'],
                                           password=bmc['password'])
            ret = ipmicmd.set_power(cmd, wait=True)
        except pyghmi_exception.IpmiException:
            msg = 'IPMI cmd {!r} failed on bmc {!r}, {!r}'.format(
                cmd, bmc['address'], host)
            LOG.error(msg, exc_info=True)
            raise

        LOG.debug('IPMI response: {}'.format(ret))
        if ret.get('powerstate') != expected_state or 'error' in ret:
            msg = ('Failed to change power state to {!r} on bmc {!r}, '
                   '{!r}'.format(expected_state, bmc['address'], host))
            raise error.PowerManagementError(msg)

    def supports(self, host):
        try:
            self._find_bmc_by_host(host)
        except error.PowerManagementError:
            return False
        return True

    def poweroff(self, host):
        LOG.debug('Power off Node: %s', host)
        self._run_set_power_cmd(host, cmd='off', expected_state='off')
        LOG.info('Node powered off: %s', host)

    def poweron(self, host):
        LOG.debug('Power on Node: %s', host)
        self._run_set_power_cmd(host, cmd='on', expected_state='on')
        LOG.info('Node powered on: %s', host)

    def reset(self, host):
        LOG.debug('Reset Node: %s', host)
        # boot -- If system is off, then 'on', else 'reset'
        self._run_set_power_cmd(host, cmd='boot')
        # NOTE(astudenov): This command does not wait for node to boot
        LOG.info('Node reset: %s', host)

    def shutdown(self, host):
        LOG.debug('Shutdown Node: %s', host)
        self._run_set_power_cmd(host, cmd='shutdown', expected_state='off')
        LOG.info('Node is off: %s', host)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from os_faults.cmd import cmd
from os_faults.tests.unit import test


class CmdTestCase(test.TestCase):

    def test_main_no_args(self):
        self.assertRaises(SystemExit, cmd.main)

    @mock.patch('os_faults.human_api')
    @mock.patch('os_faults.connect')
    def test_main_verify(self, mock_connect, mock_human_api):
        with mock.patch('sys.argv', ['', '-c', 'my_file', '-v']):
            cmd.main()

            mock_connect.assert_called_once_with(config_filename='my_file')
            mock_connect.return_value.verify.assert_called_once_with()
            mock_human_api.assert_not_called()

    @mock.patch('os_faults.human_api')
    @mock.patch('os_faults.connect')
    def test_main_command(self, mock_connect, mock_human_api):
        with mock.patch('sys.argv', ['', '-c', 'my_file', 'my_command']):
            cmd.main()

            mock_connect.assert_called_once_with(config_filename='my_file')
            mock_connect.return_value.verify.assert_not_called()
            mock_human_api.assert_called_once_with(
                mock_connect.return_value, 'my_command')

    @mock.patch('os_faults.human_api')
    @mock.patch('os_faults.connect')
    def test_main_verify_and_command(self, mock_connect, mock_human_api):
        with mock.patch('sys.argv', ['', '-c', 'my_file', '-v', 'my_command']):
            cmd.main()

            mock_connect.assert_called_once_with(config_filename='my_file')
            mock_connect.return_value.verify.assert_called_once_with()
            mock_human_api.assert_called_once_with(
                mock_connect.return_value, 'my_command')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import jsonschema
import mock

import os_faults
from os_faults.ansible import executor
from os_faults.api import cloud_management
from os_faults.api import error
from os_faults.api import node_collection
from os_faults.api import service
from os_faults.drivers.cloud import devstack
from os_faults.drivers.power import libvirt
from os_faults.tests.unit import test


class OSFaultsTestCase(test.TestCase):

    def setUp(self):
        super(OSFaultsTestCase, self).setUp()
        self.cloud_config = {
            'cloud_management': {
                'driver': 'devstack',
                'args': {
                    'address': '10.30.00.5',
                    'auth': {
                        'username': 'root',
                        'private_key_file': '/my/path/pk.key',
                    },
                }
            },
            'power_managements': [{
                'driver': 'libvirt',
                'args': {
                    'connection_uri': "qemu+ssh://user@10.30.20.21/system"
                }
            }]
        }

    @mock.patch('os_faults.ansible.executor.AnsibleRunner')
    def test_connect_devstack(self, _):
        cloud_config = {
            'cloud_management': {
                'driver': 'devstack',
                'args': {
                    'address': 'devstack.local',
                    'auth': {
                        'username': 'developer',
                        'private_key_file': '/my/path/pk.key',
                    },
                }
            }
        }
        destructor = os_faults.connect(cloud_config)
        self.assertIsInstance(destructor, devstack.DevStackCloudManagement)

    @mock.patch('os_faults.ansible.executor.AnsibleRunner')
    def test_config_with_services(self, _):
        self.cloud_config['services'] = {
            'app': {
                'driver': 'process',
                'args': {'grep': 'myapp'}
            }
        }
        destructor = os_faults.connect(self.cloud_config)
        app = destructor.get_service('app')
        self.assertIsNotNone(app)

    @mock.patch('os_faults.ansible.executor.AnsibleRunner')
    def test_config_with_services_and_hosts(self, _):
        self.cloud_config['node_discover'] = {
            'driver': 'node_list',
            'args': [
                {
                    'ip': '10.0.0.11',
                    'mac': '01:ab:cd:01:ab:cd',
                    'fqdn': 'node-1'
                }, {
                    'ip': '10.0.0.12',
                    'mac': '02:ab:cd:02:ab:cd',
                    'fqdn': 'node-2'
                },
            ]
        }
        self.cloud_config['services'] = {
            'app': {
                'driver': 'process',
                'args': {'grep': 'myapp'},
                'hosts': ['10.0.0.11', '10.0.0.12']
            }
        }
        destructor = os_faults.connect(self.cloud_config)
        app = destructor.get_service('app')
        self.assertIsNotNone(app)
        nodes = app.get_nodes()
        self.assertEqual(['10.0.0.11', '10.0.0.12'], nodes.get_ips())
        self.assertEqual(['node-1', 'node-2'], nodes.get_fqdns())
        self.assertEqual(['01:ab:cd:01:ab:cd', '02:ab:cd:02:ab:cd'],
                         nodes.get_macs())

    @mock.patch('os_faults.ansible.executor.AnsibleRunner')
    def test_connect_with_libvirt(self, _):
        destructor = os_faults.connect(self.cloud_config)
        self.assertIsInstance(destructor, devstack.DevStackCloudManagement)
        self.assertEqual(1, len(destructor.power_manager.power_drivers))
        self.assertIsInstance(destructor.power_manager.power_drivers[0],
                              libvirt.LibvirtDriver)

    def test_connect_driver_not_found(self):
        cloud_config = {
            'cloud_management': {
                'driver': 'non-existing',
                'args': {},
            }
        }
        self.assertRaises(
            error.OSFDriverNotFound, os_faults.connect, cloud_config)

    def test_connect_driver_not_specified(self):
        cloud_config = {'foo': 'bar'}
        self.assertRaises(
            jsonschema.ValidationError, os_faults.connect, cloud_config)

    @mock.patch('os.path.exists', return_value=False)
    def test_connect_no_config_files(self, mock_os_path_exists):
        self.assertRaises(error.OSFError, os_faults.connect)

    @mock.patch('os.path.exists', side_effect=lambda x: 'bad' not in x)
    @mock.patch('os.walk', side_effect=lambda x: ([x, [], []],
                                                  [x + 'subdir', [], []]))
    @mock.patch.object(executor, 'MODULE_PATHS', set())
    def test_register_ansible_modules(self, mock_os_walk, mock_os_path_exists):
        os_faults.register_ansible_modules(['/my/path/', '/other/path/'])
        self.assertEqual(executor.get_module_paths(),
                         {'/my/path/', '/my/path/subdir',
                          '/other/path/', '/other/path/subdir'})

        self.assertRaises(error.OSFError, os_faults.register_ansible_modules,
                          ['/my/bad/path/'])

    @mock.patch('os_faults.connect')
    def test_discover(self, mock_connect):
        cloud_config = {
            'cloud_management': {
                'driver': 'devstack',
                'args': {
                    'address': 'devstack.local',
                    'username': 'developer',
                    'private_key_file': '/my/path/pk.key',
                }
            }
        }
        cloud_management_mock = mock.create_autospec(
            cloud_management.CloudManagement)
        mock_connect.return_value = cloud_management_mock
        cloud_management_mock.get_nodes.return_value.hosts = [
            node_collection.Host(
                ip='10.0.0.2', mac='09:7b:74:90:63:c1', fqdn='node1.local'),
            node_collection.Host(
                ip='10.0.0.3', mac='09:7b:74:90:63:c2', fqdn='node2.local')]
        cloud_management_mock.list_supported_services.return_value = [
            'srv1', 'srv2']

        def mock_service(name, config, ips):
            m = mock.create_autospec(service.Service)
            m.NAME = name
            m.config = config
            m.get_nodes.return_value.get_ips.return_value = ips
            return m

        srv1 = mock_service('process', {'grep': 'srv1'}, [])
        srv2 = mock_service('linux_service',
                            {'grep': 'srv2', 'linux_service': 'srv2'},
                            ['10.0.0.2'])
        services = {'srv1': srv1, 'srv2': srv2}
        cloud_management_mock.get_service.side_effect = services.get

        discovered_config = os_faults.discover(cloud_config)
        self.assertEqual({
            'cloud_management': {
                'driver': 'devstack',
                'args': {
                    'address': 'devstack.local',
                    'private_key_file': '/my/path/pk.key',
                    'username': 'developer'
                }
            },
            'node_discover': {
                'driver': 'node_list',
                'args': [
                    {
                        'fqdn': 'node1.local',
                        'ip': '10.0.0.2',
                        'mac': '09:7b:74:90:63:c1'
                    }, {
                        'fqdn': 'node2.local',
                        'ip': '10.0.0.3',
                        'mac': '09:7b:74:90:63:c2'
                    }
                ]
            },
            'services': {
                'srv1': {
                    'driver': 'process',
                    'args': {'grep': 'srv1'},
                },
                'srv2': {
                    'driver': 'linux_service',
                    'args': {'grep': 'srv2', 'linux_service': 'srv2'},
                    'hosts': ['10.0.0.2']
                }
            }
        }, discovered_config)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
import copy
import logging

import jsonschema
import six

from os_faults.api import base_driver
from os_faults.api import error
from os_faults.api import node_collection
from os_faults.api import power_management
from os_faults import registry

LOG = logging.getLogger(__name__)


@six.add_metaclass(abc.ABCMeta)
class CloudManagement(base_driver.BaseDriver):
    SERVICES = {}
    CONTAINERS = {}
    SUPPORTED_NETWORKS = []
    NODE_CLS = node_collection.NodeCollection

    def __init__(self):
        self.power_manager = power_management.PowerManager()
        self.node_discover = None
        self.services = copy.deepcopy(self.SERVICES)
        self.containers = copy.deepcopy(self.CONTAINERS)

    def add_power_management(self, driver):
        self.power_manager.add_driver(driver)

    def set_node_discover(self, node_discover):
        self.node_discover = node_discover

    def update_services(self, services):
        self.services.update(services)

    def update_containers(self, containers):
        self.containers.update(containers)

    def validate_services(self):
        for service_name, serive_conf in self.services.items():
            serive_cls = registry.get_driver(serive_conf["driver"])
            jsonschema.validate(serive_conf['args'], serive_cls.CONFIG_SCHEMA)

    def validate_containers(self):
        for container_name, container_conf in self.containers.items():
            container_cls = registry.get_driver(container_conf["driver"])
            jsonschema.validate(container_conf['args'],
                                container_cls.CONFIG_SCHEMA)

    @abc.abstractmethod
    def verify(self):
        """Verify connection to the cloud.

        """

    def get_nodes(self, fqdns=None):
        """Get nodes in the cloud

        This function returns NodesCollection representing all nodes in the
        cloud or only those that has specified FQDNs.
        :param fqdns list of FQDNs or None to retrieve all nodes
        :return: NodesCollection
        """

        if self.node_discover is None:
            raise error.OSFError(
                'node_discover is not specified and "{}" '
                'driver does not support discovering'.format(self.NAME))
        hosts = self.node_discover.discover_hosts()
        nodes = self.NODE_CLS(cloud_management=self, hosts=hosts)

        if fqdns:
            LOG.debug('Trying to find nodes with FQDNs: %s', fqdns)
            nodes = nodes.filter(lambda node: node.fqdn in fqdns)
            LOG.debug('The following nodes were found: %s', nodes.hosts)
        return nodes

    def get_service(self, name):
        """Get service with specified name

        :param name: name of the service
        :return: Service
        """
        if name not in self.services:
            raise error.ServiceError(
                '{} driver does not support {!r} service'.format(
                    self.NAME.title(), name))

        config = self.services[name]
        klazz = registry.get_driver(config["driver"])
        return klazz(node_cls=self.NODE_CLS, cloud_management=self,
                     service_name=name, config=config["args"],
                     hosts=config.get('hosts'))

    def get_container(self, name):
        """Get container with specified name

        :param name: name of the container
        :return: Container
        """
        if name not in self.containers:
            raise error.ContainerError(
                '{} driver does not support {!r} container'.format(
                    self.NAME.title(), name))

        config = self.containers[name]
        klazz = registry.get_driver(config["driver"])
        return klazz(node_cls=self.NODE_CLS, cloud_management=self,
                     container_name=name, config=config["args"],
                     hosts=config.get('hosts'))

    @abc.abstractmethod
    def execute_on_cloud(self, hosts, task, raise_on_error=True):
        """Execute task on specified hosts within the cloud.

        :param hosts: List of host FQDNs
        :param task: Ansible task
        :param raise_on_error: throw exception in case of error
        :return: Ansible execution result (list of records)
        """

    @classmethod
    def list_supported_services(cls):
        """Lists all services supported by this driver

        :return: [String] list of service names
        """
        return cls.SERVICES.keys()

    @classmethod
    def list_supported_networks(cls):
        """Lists all networks supported by nodes returned by this driver

        :return: [String] list of network names
        """
        return cls.SUPPORTED_NETWORKS



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

import six

from os_faults.api import base_driver


@six.add_metaclass(abc.ABCMeta)
class NodeDiscover(base_driver.BaseDriver):
    """Node discover base driver."""

    @abc.abstractmethod
    def discover_hosts(self):
        """Discover hosts

        :returns: list of Host instances
        """



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


PORT_SCHEMA = {
    'type': 'array',
    'items': [
        {'enum': ['tcp', 'udp']},
        {'type': 'integer', 'minimum': 0, 'maximum': 65535},
    ],
    'minItems': 2,
    'maxItems': 2,
}

AUTH_SCHEMA = {
    'type': 'object',
    'properties': {
        'username': {'type': 'string'},
        'password': {'type': 'string'},
        'private_key_file': {'type': 'string'},
        'become_username': {'type': 'string'},
        'become_password': {'type': 'string'},
        'become_method': {'type': 'string'},
        'jump': {
            'type': 'object',
            'properties': {
                'host': {'type': 'string'},
                'username': {'type': 'string'},
                'private_key_file': {'type': 'string'},
            },
            'required': ['host'],
            'additionalProperties': False,
        },
    },
    'additionalProperties': False,
}

SERIAL = {'type': 'integer', 'minimum': 1}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

import click
import yaml

import os_faults
from os_faults import registry


READABLE_FILE = click.Path(dir_okay=False, readable=True, exists=True,
                           resolve_path=True)
WRITABLE_FILE = click.Path(dir_okay=False, writable=True, resolve_path=True)

config_option = click.option('-c', '--config', type=READABLE_FILE,
                             help='path to os-faults cloud connection config')


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo('Version: %s' % os_faults.get_release())
    ctx.exit()


@click.group()
@click.option('--debug', '-d', is_flag=True, help='Enable debug logs')
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help='Show version and exit.')
def main(debug):
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.DEBUG if debug else logging.INFO)


@main.command()
@config_option
def verify(config):
    """Verify connection to the cloud"""
    config = config or os_faults.get_default_config_file()
    cloud_management = os_faults.connect(config_filename=config)
    cloud_management.verify()


@main.command()
@click.argument('output', type=WRITABLE_FILE)
@config_option
def discover(config, output):
    """Discover services/nodes and save them to output config file"""
    config = config or os_faults.get_default_config_file()
    with open(config) as f:
        cloud_config = yaml.safe_load(f.read())
    discovered_config = os_faults.discover(cloud_config)
    with open(output, 'w') as f:
        f.write(yaml.safe_dump(discovered_config, default_flow_style=False))
    click.echo('Saved {}'.format(output))


@main.command()
@config_option
def nodes(config):
    """List cloud nodes"""
    config = config or os_faults.get_default_config_file()
    cloud_management = os_faults.connect(config_filename=config)
    hosts = [{'ip': host.ip, 'mac': host.mac, 'fqdn': host.fqdn}
             for host in cloud_management.get_nodes().hosts]
    click.echo(yaml.safe_dump(hosts, default_flow_style=False), nl=False)


@main.command()
def drivers():
    """List os-faults drivers"""
    drivers = sorted(registry.get_drivers().keys())
    click.echo(yaml.safe_dump(drivers, default_flow_style=False), nl=False)


if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ddt
import mock
from pyghmi import exceptions as pyghmi_exc

import os_faults
from os_faults.api import node_collection
from os_faults.drivers.power import ipmi
from os_faults import error
from os_faults.tests.unit import test


@ddt.ddt
class IPMIDriverTestCase(test.TestCase):

    def setUp(self):
        super(IPMIDriverTestCase, self).setUp()

        self.params = {
            'mac_to_bmc': {
                '00:00:00:00:00:00': {
                    'address': '55.55.55.55',
                    'username': 'foo',
                    'password': 'bar'
                }
            },
            'fqdn_to_bmc': {
                'node2.com': {
                    'address': '55.55.55.56',
                    'username': 'ham',
                    'password': 'eggs'
                }
            }
        }
        self.driver = ipmi.IPMIDriver(self.params)
        self.host = node_collection.Host(
            ip='10.0.0.2', mac='00:00:00:00:00:00', fqdn='node1.com')
        self.host2 = node_collection.Host(
            ip='10.0.0.3', mac='00:00:00:00:00:01', fqdn='node2.com')

    @ddt.data(
        {
            'mac_to_bmc': {
                '00:00:00:00:00:00': {
                    'address': '55.55.55.55',
                    'username': 'foo',
                    'password': 'bar'
                }
            }
        },
        {
            'fqdn_to_bmc': {
                'node2.com': {
                    'address': '55.55.55.56',
                    'username': 'ham',
                    'password': 'eggs'
                }
            }
        },
        {
            'mac_to_bmc': {
                '00:00:00:00:00:00': {
                    'address': '55.55.55.55',
                    'username': 'foo',
                    'password': 'bar'
                }
            },
            'fqdn_to_bmc': {
                'node2.com': {
                    'address': '55.55.55.56',
                    'username': 'ham',
                    'password': 'eggs'
                }
            }
        },
    )
    def test_init(self, config):
        os_faults._init_driver({'driver': 'ipmi', 'args': config})

    def test_supports(self):
        self.assertTrue(self.driver.supports(self.host))
        self.assertTrue(self.driver.supports(self.host2))

    def test_supports_false(self):
        host = node_collection.Host(
            ip='10.0.0.2', mac='00:00:00:00:00:01', fqdn='node1.com')
        self.assertFalse(self.driver.supports(host))

    @mock.patch('pyghmi.ipmi.command.Command')
    def test__run_set_power_cmd(self, mock_command):
        ipmicmd = mock_command.return_value
        ipmicmd.set_power.return_value = {'powerstate': 'off'}

        self.driver._run_set_power_cmd(self.host, 'off', expected_state='off')
        ipmicmd.set_power.assert_called_once_with('off', wait=True)

    @mock.patch('pyghmi.ipmi.command.Command')
    def test__run_set_power_cmd_ipmi_exc(self, mock_command):
        ipmicmd = mock_command.return_value
        ipmicmd.set_power.side_effect = pyghmi_exc.IpmiException()

        self.assertRaises(pyghmi_exc.IpmiException,
                          self.driver._run_set_power_cmd,
                          self.host, 'off', expected_state='off')

    @mock.patch('pyghmi.ipmi.command.Command')
    def test__run_set_power_cmd_unexpected_power_state(self, mock_command):
        ipmicmd = mock_command.return_value
        ipmicmd.set_power.return_value = {'powerstate': 'unexpected state'}

        self.assertRaises(error.PowerManagementError,
                          self.driver._run_set_power_cmd,
                          self.host, 'off', expected_state='off')

    @mock.patch('os_faults.drivers.power.ipmi.IPMIDriver._run_set_power_cmd')
    @ddt.data(('poweroff', 'off', 'off'),
              ('poweron', 'on', 'on'),
              ('reset', 'boot'),
              ('shutdown', 'shutdown', 'off'))
    def test_driver_actions(self, actions, mock__run_set_power_cmd):
        getattr(self.driver, actions[0])(self.host)
        if len(actions) == 3:
            mock__run_set_power_cmd.assert_called_once_with(
                self.host, cmd=actions[1], expected_state=actions[2])
        else:
            mock__run_set_power_cmd.assert_called_once_with(
                self.host, cmd=actions[1])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os_faults.drivers.services import process
from os_faults.drivers import shared_schemas


class SystemService(process.ServiceAsProcess):
    """System service

    This is universal driver for any system services supported by Ansible
    (e.g. systemd, upstart). Please refer to Ansible documentation
    http://docs.ansible.com/ansible/latest/service_module.html for the
    whole list.

    **Example configuration:**

    .. code-block:: yaml

        services:
          app:
            driver: system_service
            args:
              service_name: app
              grep: my_app
              port: ['tcp', 4242]

    parameters:

    - **service_name** - name of a service
    - **grep** - regexp for grep to find process PID
    - **port** - tuple with two values - protocol, port number (optional)
    """
    NAME = 'system_service'
    DESCRIPTION = 'System Service (systemd, upstart, SysV, etc.)'
    CONFIG_SCHEMA = {
        'type': 'object',
        'properties': {
            'service_name': {'type': 'string'},
            'grep': {'type': 'string'},
            'port': shared_schemas.PORT_SCHEMA,
        },
        'required': ['grep', 'service_name'],
        'additionalProperties': False,
    }

    def __init__(self, *args, **kwargs):
        super(SystemService, self).__init__(*args, **kwargs)
        self.service_name = self.config['service_name']

    def start(self, nodes=None):
        task = {
            'service': {
                'name': self.service_name, 'state': 'started'
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, 'Start')

    def terminate(self, nodes=None):
        task = {
            'service': {
                'name': self.service_name, 'state': 'stopped',
                'pattern': self.grep,
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, 'Terminate')

    def restart(self, nodes=None):
        task = {
            'service': {
                'name': self.service_name, 'state': 'restarted'
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, 'Restart')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import yaml

from os_faults.ansible import executor
from os_faults.api import cloud_management
from os_faults.api import node_collection
from os_faults.api import node_discover
from os_faults.drivers import shared_schemas
from os_faults import error

LOG = logging.getLogger(__name__)


class SaltCloudNodeCollection(node_collection.NodeCollection):

    def connect(self, network_name):
        raise NotImplementedError

    def disconnect(self, network_name):
        raise NotImplementedError


class SaltCloudManagement(cloud_management.CloudManagement,
                          node_discover.NodeDiscover):
    """Driver for OpenStack cloud managed by Salt.

    Supports discovering of slave nodes.

    **Example configuration:**

    .. code-block:: yaml

        cloud_management:
          driver: saltcloud
          args:
            address: 192.168.1.10
            auth:
              username: root
              password: root_pass
              private_key_file: ~/.ssh/id_rsa_tcpcloud
            slave_auth:
              username: ubuntu
              password: ubuntu_pass
              become_username: root
            slave_name_regexp: ^(?!cfg|mon)
            slave_direct_ssh: True
            get_ips_cmd: pillar.get _param:single_address

    parameters:

    - **address** - ip address of salt config node
    - **username** - username for salt config node
    - **password** - password for salt config node (optional)
    - **private_key_file** - path to key file (optional)
    - **slave_username** - username for salt minions (optional) *username*
      will be used if *slave_username* not specified
    - **slave_password** - password for salt minions (optional) *password*
      will be used if *slave_password* not specified
    - **master_sudo** - Use sudo on salt config node (optional)
    - **slave_sudo** - Use sudo on salt minion nodes (optional)
    - **slave_name_regexp** - regexp for minion FQDNs (optional)
    - **slave_direct_ssh** - if *False* then salt master is used as ssh proxy
      (optional)
    - **get_ips_cmd** - salt command to get IPs of minions (optional)
    - **serial** - how many hosts Ansible should manage at a single time.
      (optional) default: 10
    """

    NAME = 'saltcloud'
    DESCRIPTION = 'SaltCloud management driver'
    NODE_CLS = SaltCloudNodeCollection
    SERVICES = {
        'keystone': {
            'driver': 'salt_service',
            'args': {
                'grep': 'keystone-all',
                'salt_service': 'keystone',
            }
        },
        'horizon': {
            'driver': 'salt_service',
            'args': {
                'grep': 'apache2',
                'salt_service': 'apache2',
            }
        },
        'memcached': {
            'driver': 'salt_service',
            'args': {
                'grep': 'memcached',
                'salt_service': 'memcached',
            }
        },
        'mysql': {
            'driver': 'salt_service',
            'args': {
                'grep': 'mysqld',
                'salt_service': 'mysql',
                'port': ['tcp', 3307],
            }
        },
        'rabbitmq': {
            'driver': 'salt_service',
            'args': {
                'grep': r'beam\.smp .*rabbitmq_server',
                'salt_service': 'rabbitmq-server',
            }
        },
        'glance-api': {
            'driver': 'salt_service',
            'args': {
                'grep': 'glance-api',
                'salt_service': 'glance-api',
            }
        },
        'glance-glare': {
            'driver': 'salt_service',
            'args': {
                'grep': 'glance-glare',
                'salt_service': 'glance-glare',
            }
        },
        'glance-registry': {
            'driver': 'salt_service',
            'args': {
                'grep': 'glance-registry',
                'salt_service': 'glance-registry',
            }
        },
        'nova-api': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-api',
                'salt_service': 'nova-api',
            }
        },
        'nova-compute': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-compute',
                'salt_service': 'nova-compute',
            }
        },
        'nova-scheduler': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-scheduler',
                'salt_service': 'nova-scheduler',
            }
        },
        'nova-cert': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-cert',
                'salt_service': 'nova-cert',
            }
        },
        'nova-conductor': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-conductor',
                'salt_service': 'nova-conductor',
            }
        },
        'nova-consoleauth': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-consoleauth',
                'salt_service': 'nova-consoleauth',
            }
        },
        'nova-novncproxy': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nova-novncproxy',
                'salt_service': 'nova-novncproxy',
            }
        },
        'neutron-server': {
            'driver': 'salt_service',
            'args': {
                'grep': 'neutron-server',
                'salt_service': 'neutron-server',
            }
        },
        'neutron-dhcp-agent': {
            'driver': 'salt_service',
            'args': {
                'grep': 'neutron-dhcp-agent',
                'salt_service': 'neutron-dhcp-agent',
            }
        },
        'neutron-metadata-agent': {
            'driver': 'salt_service',
            'args': {
                'grep': 'neutron-metadata-agent',
                'salt_service': 'neutron-metadata-agent',
            }
        },
        'neutron-openvswitch-agent': {
            'driver': 'salt_service',
            'args': {
                'grep': 'neutron-openvswitch-agent',
                'salt_service': 'neutron-openvswitch-agent',
            }
        },
        'neutron-l3-agent': {
            'driver': 'salt_service',
            'args': {
                'grep': 'neutron-l3-agent',
                'salt_service': 'neutron-l3-agent',
            }
        },
        'heat-api': {
            'driver': 'salt_service',
            'args': {
                # space at the end filters heat-api-* services
                'grep': 'heat-api ',
                'salt_service': 'heat-api',
            }
        },
        'heat-engine': {
            'driver': 'salt_service',
            'args': {
                'grep': 'heat-engine',
                'salt_service': 'heat-engine',
            }
        },
        'cinder-api': {
            'driver': 'salt_service',
            'args': {
                'grep': 'cinder-api',
                'salt_service': 'cinder-api',
            }
        },
        'cinder-scheduler': {
            'driver': 'salt_service',
            'args': {
                'grep': 'cinder-scheduler',
                'salt_service': 'cinder-scheduler',
            }
        },
        'cinder-volume': {
            'driver': 'salt_service',
            'args': {
                'grep': 'cinder-volume',
                'salt_service': 'cinder-volume',
            }
        },
        'cinder-backup': {
            'driver': 'salt_service',
            'args': {
                'grep': 'cinder-backup',
                'salt_service': 'cinder-backup',
            }
        },
        'elasticsearch': {
            'driver': 'salt_service',
            'args': {
                'grep': 'java .*elasticsearch',
                'salt_service': 'elasticsearch',
            }
        },
        'grafana-server': {
            'driver': 'salt_service',
            'args': {
                'grep': 'grafana-server',
                'salt_service': 'grafana-server',
            }
        },
        'influxdb': {
            'driver': 'salt_service',
            'args': {
                'grep': 'influxd',
                'salt_service': 'influxdb',
            }
        },
        'kibana': {
            'driver': 'salt_service',
            'args': {
                'grep': 'kibana',
                'salt_service': 'kibana',
            }
        },
        'nagios3': {
            'driver': 'salt_service',
            'args': {
                'grep': 'nagios3',
                'salt_service': 'nagios3',
            }
        },
    }
    SUPPORTED_NETWORKS = []
    CONFIG_SCHEMA = {
        'type': 'object',
        '$schema': 'http://json-schema.org/draft-04/schema#',
        'properties': {
            'address': {'type': 'string'},
            'auth': shared_schemas.AUTH_SCHEMA,
            'slave_auth': shared_schemas.AUTH_SCHEMA,
            'slave_name_regexp': {'type': 'string'},
            'slave_direct_ssh': {'type': 'boolean'},
            'get_ips_cmd': {'type': 'string'},
            'serial': {'type': 'integer', 'minimum': 1},
        },
        'required': ['address', 'username'],
        'additionalProperties': False,
    }

    def __init__(self, cloud_management_params):
        super(SaltCloudManagement, self).__init__()
        self.node_discover = self  # supports discovering

        self.master_node_address = cloud_management_params['address']
        self._master_host = node_collection.Host(ip=self.master_node_address)
        self.slave_direct_ssh = cloud_management_params.get(
            'slave_direct_ssh', False)
        use_jump = not self.slave_direct_ssh
        self.get_ips_cmd = cloud_management_params.get(
            'get_ips_cmd', 'pillar.get _param:single_address')
        self.serial = cloud_management_params.get('serial')

        self.master_node_executor = executor.AnsibleRunner(
            auth=cloud_management_params.get('auth'))

        slave_auth = cloud_management_params.get('slave_auth') or {}
        if use_jump:
            slave_auth['jump'] = {}
            jump = slave_auth['jump']
            jump['host'] = self.master_node_address
            if not jump.get('username'):
                jump['username'] = (
                    slave_auth.get('username') or
                    cloud_management_params['auth']['username'])

        self.cloud_executor = executor.AnsibleRunner(
            auth=slave_auth,
            serial=self.serial)

        # get all nodes except salt master (that has cfg* hostname) by default
        self.slave_name_regexp = cloud_management_params.get(
            'slave_name_regexp', '^(?!cfg|mon)')

        self.cached_cloud_hosts = list()

    def verify(self):
        """Verify connection to the cloud."""
        nodes = self.get_nodes()
        LOG.debug('Cloud nodes: %s', nodes)

        task = {'command': 'hostname'}
        task_result = self.execute_on_cloud(nodes.hosts, task)
        LOG.debug('Hostnames of cloud nodes: %s',
                  [r.payload['stdout'] for r in task_result])

        LOG.info('Connected to cloud successfully!')

    def _run_salt(self, command):
        cmd = "salt -E '{}' {} --out=yaml".format(
            self.slave_name_regexp, command)
        result = self._execute_on_master_node({'command': cmd})
        return yaml.safe_load(result[0].payload['stdout'])

    def discover_hosts(self):
        if not self.cached_cloud_hosts:
            interfaces = self._run_salt("network.interfaces")
            ips = self._run_salt(self.get_ips_cmd)

            for fqdn, ip in ips.items():
                node_ifaces = interfaces[fqdn]

                mac = None
                for iface_name, net_data in node_ifaces.items():
                    iface_ips = [data['address']
                                 for data in net_data.get('inet', [])]
                    if ip in iface_ips:
                        mac = net_data['hwaddr']
                        break
                else:
                    raise error.CloudManagementError(
                        "Can't find ip {} on node {} with node_ifaces:\n{}"
                        "".format(ip, fqdn, yaml.dump(node_ifaces)))

                host = node_collection.Host(ip=ip, mac=mac, fqdn=fqdn)
                self.cached_cloud_hosts.append(host)
            self.cached_cloud_hosts = sorted(self.cached_cloud_hosts)

        return self.cached_cloud_hosts

    def _execute_on_master_node(self, task):
        """Execute task on salt master node.

        :param task: Ansible task
        :return: Ansible execution result (list of records)
        """
        return self.master_node_executor.execute([self._master_host], task)

    def execute_on_cloud(self, hosts, task, raise_on_error=True):
        """Execute task on specified hosts within the cloud.

        :param hosts: List of host FQDNs
        :param task: Ansible task
        :param raise_on_error: throw exception in case of error
        :return: Ansible execution result (list of records)
        """
        if raise_on_error:
            return self.cloud_executor.execute(hosts, task)
        else:
            return self.cloud_executor.execute(hosts, task, [])



#!/usr/bin/python

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ansible.module_utils.basic import AnsibleModule  # noqa


def main():
    module = AnsibleModule(
        argument_spec=dict(
            service=dict(required=True, type='str'),
            action=dict(required=True, choices=['block', 'unblock']),
            port=dict(required=True, type='int'),
            protocol=dict(required=True, choices=['tcp', 'udp']),
        ))

    service = module.params['service']
    action = module.params['action']
    port = module.params['port']
    protocol = module.params['protocol']
    comment = '{}_temporary_DROP'.format(service)

    if action == 'block':
        cmd = ('bash -c "iptables -I INPUT 1 -p {protocol} --dport {port} '
               '-j DROP -m comment --comment "{comment}""'.format(
                   comment=comment, port=port, protocol=protocol))
    else:
        cmd = ('bash -c "rule=`iptables -L INPUT -n --line-numbers | '
               'grep "{comment}" | cut -d \' \' -f1`; for arg in $rule;'
               ' do iptables -D INPUT -p {protocol} --dport {port} '
               '-j DROP -m comment --comment "{comment}"; done"'.format(
                   comment=comment, port=port, protocol=protocol))
    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
    module.exit_json(cmd=cmd, rc=rc, stderr=stderr, stdout=stdout)


if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from os_faults.api import error
from os_faults.api import node_collection
from os_faults.api import power_management
from os_faults.tests.unit import test


class PowerManagerTestCase(test.TestCase):

    def setUp(self):
        super(PowerManagerTestCase, self).setUp()

        self.dummy_driver1 = mock.Mock(spec=power_management.PowerDriver)
        self.dummy_driver1.supports.side_effect = lambda host: 'c1' in host.mac

        self.dummy_driver2 = mock.Mock(spec=power_management.PowerDriver)
        self.dummy_driver2.supports.side_effect = lambda host: 'c2' in host.mac

        self.dummy_drivers = [self.dummy_driver1, self.dummy_driver2]
        self.hosts = [
            node_collection.Host(ip='10.0.0.2', mac='09:7b:74:90:63:c1',
                                 fqdn='node1.com'),
            node_collection.Host(ip='10.0.0.3', mac='09:7b:74:90:63:c2',
                                 fqdn='node2.com'),
        ]
        self.pm = power_management.PowerManager()
        self.pm.add_driver(self.dummy_driver1)
        self.pm.add_driver(self.dummy_driver2)

    def test_poweroff(self):
        self.pm.poweroff(self.hosts)

        self.dummy_driver1.poweroff.called_once_with(host=self.hosts[0])
        self.dummy_driver2.poweroff.called_once_with(host=self.hosts[1])

    def test_poweron(self):
        self.pm.poweron(self.hosts)

        self.dummy_driver1.poweron.called_once_with(host=self.hosts[0])
        self.dummy_driver2.poweron.called_once_with(host=self.hosts[1])

    def test_reset(self):
        self.pm.reset(self.hosts)

        self.dummy_driver1.reset.called_once_with(host=self.hosts[0])
        self.dummy_driver2.reset.called_once_with(host=self.hosts[1])

    def test_shutdown(self):
        self.pm.shutdown(self.hosts)

        self.dummy_driver1.shutdown.called_once_with(host=self.hosts[0])
        self.dummy_driver2.shutdown.called_once_with(host=self.hosts[1])

    def test_snapshot(self):
        self.pm.snapshot(self.hosts, 'snap1', suspend=False)

        self.dummy_driver1.snapshot.called_once_with(host=self.hosts[0],
                                                     snapshot_name='snap1',
                                                     suspend=False)
        self.dummy_driver2.snapshot.called_once_with(host=self.hosts[1],
                                                     snapshot_name='snap1',
                                                     suspend=False)

    def test_revert(self):
        self.pm.revert(self.hosts, 'snap1', resume=False)

        self.dummy_driver1.revert.called_once_with(host=self.hosts[0],
                                                   snapshot_name='snap1',
                                                   resume=False)
        self.dummy_driver2.revert.called_once_with(host=self.hosts[1],
                                                   snapshot_name='snap1',
                                                   resume=False)

    def test_run_error(self):
        self.dummy_driver2.reset.side_effect = Exception()

        exc = self.assertRaises(error.PowerManagementError,
                                self.pm.reset, self.hosts)
        self.assertEqual("There are some errors when working the driver. "
                         "Please, check logs for more details.", str(exc))

    def test_run_no_supported_driver(self):
        self.dummy_driver2.supports.side_effect = None
        self.dummy_driver2.supports.return_value = False

        exc = self.assertRaises(error.PowerManagementError,
                                self.pm.reset, self.hosts)
        self.assertEqual("No supported driver found for host "
                         "Host(ip='10.0.0.3', mac='09:7b:74:90:63:c2', "
                         "fqdn='node2.com', libvirt_name=None)", str(exc))

    def test_run_no_drivers(self):
        self.pm = power_management.PowerManager()

        exc = self.assertRaises(error.PowerManagementError,
                                self.pm.reset, self.hosts)
        self.assertEqual("No supported driver found for host "
                         "Host(ip='10.0.0.2', mac='09:7b:74:90:63:c1', "
                         "fqdn='node1.com', libvirt_name=None)", str(exc))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ddt
import mock

from os_faults.api import error
from os_faults.api import human
from os_faults.api import node_collection
from os_faults.api import service as service_api
from os_faults.tests.unit import test


@ddt.ddt
class TestHumanAPI(test.TestCase):
    def setUp(self):
        super(TestHumanAPI, self).setUp()
        self.destructor = mock.MagicMock()
        self.service = mock.MagicMock(service_api.Service)

        self.destructor.get_service = mock.MagicMock(return_value=self.service)

    @ddt.data(('restart', 'keystone'), ('kill', 'nova-api'))
    @ddt.unpack
    def test_service_action(self, action, service_name):

        command = '%s %s service' % (action, service_name)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        getattr(self.service, action).assert_called_once_with()

    @ddt.data(('restart', 'keystone', 'random'), ('kill', 'nova-api', 'one'))
    @ddt.unpack
    def test_service_action_on_random_node(self, action, service_name, node):

        nodes = mock.MagicMock(node_collection.NodeCollection)
        self.service.get_nodes = mock.MagicMock(return_value=nodes)

        one_node = mock.MagicMock(node_collection.NodeCollection)
        nodes.pick = mock.MagicMock(return_value=one_node)

        command = '%s %s service on %s node' % (action, service_name, node)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        getattr(self.service, action).assert_called_once_with(nodes=one_node)

        nodes.pick.assert_called_once()

    @ddt.data(('freeze', 'keystone', 5))
    @ddt.unpack
    def test_service_action_with_duration(self, action, service_name, t):

        command = '%s %s service for %d seconds' % (action, service_name, t)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        getattr(self.service, action).assert_called_once_with(sec=t)

    @ddt.data(('restart', 'keystone', 'node'), ('kill', 'nova-api', 'node'))
    @ddt.unpack
    def test_service_action_on_fqdn_node(self, action, service_name, node):

        nodes = mock.MagicMock(node_collection.NodeCollection)
        self.destructor.get_nodes.return_value = nodes

        command = '%s %s service on %s node' % (action, service_name, node)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        self.destructor.get_nodes.assert_called_once_with(fqdns=[node])
        getattr(self.service, action).assert_called_once_with(nodes=nodes)

    @ddt.data(('reboot', 'keystone'), ('reset', 'nova-api'))
    @ddt.unpack
    def test_node_action_on_all_nodes(self, action, service_name):

        nodes = mock.MagicMock(node_collection.NodeCollection)

        self.service.get_nodes = mock.MagicMock(return_value=nodes)

        command = '%s node with %s service' % (action, service_name)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        getattr(nodes, action).assert_called_once_with()

    @ddt.data(('reboot', 'keystone'), ('reset', 'nova-api'))
    @ddt.unpack
    def test_node_action_on_random_node(self, action, service_name):

        nodes = mock.MagicMock(node_collection.NodeCollection)
        nodes2 = mock.MagicMock(node_collection.NodeCollection)

        self.service.get_nodes = mock.MagicMock(return_value=nodes)
        nodes.pick = mock.MagicMock(return_value=nodes2)

        command = '%s one node with %s service' % (action, service_name)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        nodes.pick.assert_called_once()
        getattr(nodes2, action).assert_called_once_with()

    @ddt.data('reboot', 'poweroff', 'poweron')
    def test_node_action_by_fqdn(self, action):
        destructor = mock.MagicMock()
        nodes = mock.MagicMock(node_collection.NodeCollection)
        destructor.get_nodes = mock.MagicMock(return_value=nodes)

        command = '%s node-2.local node' % action.capitalize()
        human.execute(destructor, command)

        destructor.get_nodes.assert_called_once_with(fqdns=['node-2.local'])
        getattr(nodes, action).assert_called_once()

    @ddt.data('cpu', 'memory', 'disk', 'kernel')
    def test_stress_by_fqdn(self, target):
        action = 'stress'
        duration = 20
        destructor = mock.MagicMock()
        nodes = mock.MagicMock(node_collection.NodeCollection)
        destructor.get_nodes = mock.MagicMock(return_value=nodes)

        command = 'stress %s for %d seconds on node-2.local node' % (
            target, duration)
        human.execute(destructor, command)

        destructor.get_nodes.assert_called_once_with(fqdns=['node-2.local'])
        getattr(nodes, action).assert_called_once_with(
            target=target, duration=duration)

    @ddt.data('cpu', 'memory', 'disk', 'kernel')
    def test_stress_target(self, target):
        action = 'stress'
        duration = 20
        destructor = mock.MagicMock()
        nodes = mock.MagicMock(node_collection.NodeCollection)
        destructor.get_nodes = mock.MagicMock(return_value=nodes)

        command = 'stress %s for %d seconds on nodes' % (target, duration)
        human.execute(destructor, command)

        destructor.get_nodes.assert_called_once_with()

        getattr(nodes, action).assert_called_once_with(
            target=target, duration=duration)

    @ddt.data(('CPU', 'cpu', 10, 'keystone'),
              ('disk', 'disk', 20, 'nova-api'))
    @ddt.unpack
    def test_stress_by_service_on_fqdn_node(self, user_target, cmd_target,
                                            duration, service_name):
        action = 'stress'
        nodes = mock.MagicMock(node_collection.NodeCollection)
        self.service.get_nodes.return_value = nodes

        command = 'stress %s for %d seconds on all nodes with %s service' % (
            user_target, duration, service_name)
        human.execute(self.destructor, command)

        getattr(nodes, action).assert_called_once_with(
            target=cmd_target, duration=duration)

    @ddt.data(('Disconnect', 'disconnect'),
              ('Connect', 'connect'))
    @ddt.unpack
    def test_network_on_nodes_by_fqdn(self, user_action, action):
        destructor = mock.MagicMock()
        nodes = mock.MagicMock(node_collection.NodeCollection)
        destructor.get_nodes = mock.MagicMock(return_value=nodes)

        command = '%s storage network on node-2.local node' % user_action
        human.execute(destructor, command)

        destructor.get_nodes.assert_called_once_with(fqdns=['node-2.local'])
        getattr(nodes, action).assert_called_once_with(network_name='storage')

    @ddt.data(('disconnect', 'storage', 'mysql'),
              ('connect', 'management', 'rabbitmq'))
    @ddt.unpack
    def test_network_on_nodes_by_service(
            self, action, network_name, service_name):
        nodes = mock.MagicMock(node_collection.NodeCollection)

        self.service.get_nodes = mock.MagicMock(return_value=nodes)

        command = '%s %s network on node with %s service' % (
            action, network_name, service_name)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        self.service.get_nodes.assert_called_once()
        getattr(nodes, action).assert_called_once_with(
            network_name=network_name)

    @ddt.data(('disconnect', 'storage', 'one', 'mysql'),
              ('connect', 'management', 'random', 'rabbitmq'))
    @ddt.unpack
    def test_network_on_nodes_by_service_picked_node(
            self, action, network_name, node, service_name):
        nodes = mock.MagicMock(node_collection.NodeCollection)
        nodes2 = mock.MagicMock(node_collection.NodeCollection)

        self.service.get_nodes = mock.MagicMock(return_value=nodes)
        nodes.pick = mock.MagicMock(return_value=nodes2)

        command = '%s %s network on %s node with %s service' % (
            action, network_name, node, service_name)
        human.execute(self.destructor, command)

        self.destructor.get_service.assert_called_once_with(name=service_name)
        self.service.get_nodes.assert_called_once()
        nodes.pick.assert_called_once()
        getattr(nodes2, action).assert_called_once_with(
            network_name=network_name)

    def test_malformed_query(self):
        destructor = mock.MagicMock()

        command = 'inject some fault'
        self.assertRaises(error.OSFException, human.execute, destructor,
                          command)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from click import testing
import mock

from os_faults.api import cloud_management
from os_faults.api import node_collection
from os_faults.cmd import main
from os_faults.tests.unit import test


class MainTestCase(test.TestCase):

    def setUp(self):
        super(MainTestCase, self).setUp()
        self.runner = testing.CliRunner()

    def test_version(self):
        result = self.runner.invoke(main.main, ['--version'])
        self.assertEqual(0, result.exit_code)
        self.assertIn('Version', result.output)

    @mock.patch('os_faults.connect')
    def test_verify(self, mock_connect):
        with self.runner.isolated_filesystem():
            with open('my.yaml', 'w') as f:
                f.write('foo')
            result = self.runner.invoke(main.main, ['verify'],
                                        env={'OS_FAULTS_CONFIG': 'my.yaml'})
        self.assertEqual(0, result.exit_code)
        mock_connect.assert_called_once_with(config_filename='my.yaml')
        destructor = mock_connect.return_value
        destructor.verify.assert_called_once_with()

    @mock.patch('os_faults.connect')
    def test_verify_with_config(self, mock_connect):
        with self.runner.isolated_filesystem():
            with open('my.yaml', 'w') as f:
                f.write('foo')
            myconf = os.path.abspath(f.name)
            result = self.runner.invoke(main.main, ['verify', '-c', myconf])
        self.assertEqual(0, result.exit_code)
        mock_connect.assert_called_once_with(config_filename=myconf)
        destructor = mock_connect.return_value
        destructor.verify.assert_called_once_with()

    @mock.patch('os_faults.discover')
    def test_discover(self, mock_discover):
        mock_discover.return_value = {'foo': 'bar'}
        with self.runner.isolated_filesystem():
            with open('my.yaml', 'w') as f:
                f.write('foo')
            myconf = os.path.abspath(f.name)
            result = self.runner.invoke(main.main, ['discover', '-c', myconf,
                                                    'my-new.yaml'])
            self.assertEqual(0, result.exit_code)
            mock_discover.assert_called_once_with('foo')

            with open('my-new.yaml') as f:
                self.assertEqual('foo: bar\n', f.read())

    @mock.patch('os_faults.connect')
    def test_nodes(self, mock_connect):
        cloud_management_mock = mock.create_autospec(
            cloud_management.CloudManagement)
        mock_connect.return_value = cloud_management_mock
        cloud_management_mock.get_nodes.return_value.hosts = [
            node_collection.Host(
                ip='10.0.0.2', mac='09:7b:74:90:63:c1', fqdn='node1.local'),
            node_collection.Host(
                ip='10.0.0.3', mac='09:7b:74:90:63:c2', fqdn='node2.local')]

        with self.runner.isolated_filesystem():
            with open('my.yaml', 'w') as f:
                f.write('foo')
            myconf = os.path.abspath(f.name)
            result = self.runner.invoke(main.main, ['nodes', '-c', myconf])
        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            '- fqdn: node1.local\n'
            '  ip: 10.0.0.2\n'
            '  mac: 09:7b:74:90:63:c1\n'
            '- fqdn: node2.local\n'
            '  ip: 10.0.0.3\n'
            '  mac: 09:7b:74:90:63:c2\n', result.output)

    @mock.patch('os_faults.registry.get_drivers')
    def test_drivers(self, mock_get_drivers):
        mock_get_drivers.return_value = {'foo': 1, 'bar': 2}

        with self.runner.isolated_filesystem():
            result = self.runner.invoke(main.main, ['drivers'])
        self.assertEqual(0, result.exit_code)
        self.assertEqual(
            '- bar\n'
            '- foo\n', result.output)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import threading

import mock

from os_faults.tests.unit import test
from os_faults import utils


class MyException(Exception):
    pass


class UtilsTestCase(test.TestCase):

    def test_start_thread(self):
        target = mock.Mock()
        target_params = {'param1': 'val1', 'param2': 'val2'}

        tw = utils.ThreadsWrapper()
        tw.start_thread(target, **target_params)
        tw.join_threads()

        target.assert_has_calls([mock.call(param1='val1', param2='val2')])
        self.assertIsInstance(tw.threads[0], threading.Thread)
        self.assertEqual(len(tw.errors), 0)

    def test_start_thread_raise_exception(self):
        target = mock.Mock()
        target.side_effect = MyException()

        tw = utils.ThreadsWrapper()
        tw.start_thread(target)
        tw.join_threads()

        self.assertEqual(type(tw.errors[0]), MyException)

    def test_join_threads(self):
        thread_1 = mock.Mock()
        thread_2 = mock.Mock()

        tw = utils.ThreadsWrapper()
        tw.threads = [thread_1, thread_2]
        tw.join_threads()

        thread_1.join.assert_called_once()
        thread_2.join.assert_called_once()


class MyClass(object):
    FOO = 10

    def __init__(self):
        self.BAR = None

    @utils.require_variables('FOO')
    def method(self, a, b):
        return self.FOO + a + b

    @utils.require_variables('BAR', 'BAZ')
    def method_that_miss_variables(self):
        return self.BAR, self.BAZ


class RequiredVariablesTestCase(test.TestCase):

    def test_require_variables(self):
        inst = MyClass()
        self.assertEqual(inst.method(1, b=2), 13)

    def test_require_variables_not_implemented(self):
        inst = MyClass()
        err = self.assertRaises(NotImplementedError,
                                inst.method_that_miss_variables)
        msg = 'BAR, BAZ required for MyClass.method_that_miss_variables'
        self.assertEqual(str(err), msg)


class MyPoint(utils.ComparableMixin):
    ATTRS = ('a', 'b')

    def __init__(self, a, b):
        self.a = a
        self.b = b


class ComparableMixinTestCase(test.TestCase):

    def test_operations(self):
        p1 = MyPoint(1, 'a')
        p2 = MyPoint(1, 'b')
        p3 = MyPoint(2, 'c')
        p4 = MyPoint(2, 'c')

        self.assertTrue(p1 < p2)
        self.assertTrue(p1 <= p2)
        self.assertFalse(p1 == p2)
        self.assertFalse(p1 >= p2)
        self.assertFalse(p1 > p2)
        self.assertTrue(p1 != p2)
        self.assertTrue(hash(p1) != hash(p2))

        self.assertTrue(p2 < p3)
        self.assertTrue(p2 <= p3)
        self.assertFalse(p2 == p3)
        self.assertFalse(p2 >= p3)
        self.assertFalse(p2 > p3)
        self.assertTrue(p2 != p3)
        self.assertTrue(hash(p2) != hash(p3))

        self.assertFalse(p3 < p4)
        self.assertTrue(p3 <= p4)
        self.assertTrue(p3 == p4)
        self.assertTrue(p3 >= p4)
        self.assertFalse(p3 > p4)
        self.assertFalse(p3 != p4)
        self.assertEqual(hash(p3), hash(p4))


class MyRepr(utils.ReprMixin):
    REPR_ATTRS = ('a', 'b', 'c')

    def __init__(self):
        self.a = 'foo'
        self.b = {'foo': 'bar'}
        self.c = 42


class ReprMixinTestCase(test.TestCase):

    def test_repr(self):
        r = MyRepr()
        self.assertEqual("MyRepr(a='foo', b={'foo': 'bar'}, c=42)", repr(r))



#!/usr/bin/python

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ansible.module_utils.basic import AnsibleModule  # noqa


def main():
    module = AnsibleModule(
        argument_spec=dict(
            grep=dict(required=True, type='str'),
            sig=dict(required=True, type='int')
        ))

    grep = module.params['grep']
    sig = module.params['sig']

    cmd = ('bash -c "ps ax | grep -v grep | grep \'%s\' | awk {\'print $1\'} '
           '| xargs kill -%s"') % (grep, sig)
    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
    module.exit_json(cmd=cmd, rc=rc, stderr=stderr, stdout=stdout)


if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import os

import appdirs
import jsonschema
import logging
import pbr.version
import yaml

from os_faults.ansible import executor
from os_faults.api import error
from os_faults.api import human
from os_faults import registry

LOG = logging.getLogger(__name__)

# Set default logging handler to avoid "No handler found" warnings.
LOG.addHandler(logging.NullHandler())


def get_version():
    return pbr.version.VersionInfo('os_faults').version_string()


def get_release():
    return pbr.version.VersionInfo('os_faults').release_string()


APPDIRS = appdirs.AppDirs(appname='openstack', appauthor='OpenStack')
UNIX_SITE_CONFIG_HOME = '/etc/openstack'
CONFIG_SEARCH_PATH = [
    os.getcwd(),
    APPDIRS.user_config_dir,
    UNIX_SITE_CONFIG_HOME,
]
CONFIG_FILES = [
    os.path.join(d, 'os-faults' + s)
    for d in CONFIG_SEARCH_PATH
    for s in ['.json', '.yaml', '.yml']
]

CONFIG_SCHEMA = {
    'type': 'object',
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'properties': {
        'node_discover': {
            'type': 'object',
            'properties': {
                'driver': {'type': 'string'},
                'args': {},
            },
            'required': ['driver', 'args'],
            'additionalProperties': False,
        },
        'services': {
            'type': 'object',
            'patternProperties': {
                '.*': {
                    'type': 'object',
                    'properties': {
                        'driver': {'type': 'string'},
                        'args': {'type': 'object'},
                        'hosts': {
                            'type': 'array',
                            'minItems': 1,
                            'items': {'type': 'string'},
                        },
                    },
                    'required': ['driver', 'args'],
                    'additionalProperties': False,
                }
            },
            'additionalProperties': False,
        },
        'cloud_management': {
            'type': 'object',
            'properties': {
                'driver': {'type': 'string'},
                'args': {'type': 'object'},
            },
            'required': ['driver'],
            'additionalProperties': False,
        },
        'power_managements': {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'driver': {'type': 'string'},
                    'args': {'type': 'object'},
                },
                'required': ['driver', 'args'],
                'additionalProperties': False,
            },
            'minItems': 1,
        },
    },
    'required': ['cloud_management'],
}


def get_default_config_file():
    if 'OS_FAULTS_CONFIG' in os.environ:
        return os.environ['OS_FAULTS_CONFIG']

    for config_file in CONFIG_FILES:
        if os.path.exists(config_file):
            return config_file

    msg = 'Config file is not found on any of paths: {}'.format(CONFIG_FILES)
    raise error.OSFError(msg)


def _init_driver(params):
    driver_cls = registry.get_driver(params['driver'])

    args = params.get('args') or {}  # driver may have no arguments
    if args:
        jsonschema.validate(args, driver_cls.CONFIG_SCHEMA)
    return driver_cls(args)


def connect(cloud_config=None, config_filename=None):
    """Connects to the cloud

    :param cloud_config: dict with cloud and power management params
    :param config_filename: name of the file where to read config from
    :returns: CloudManagement object
    """
    if cloud_config is None:
        config_filename = config_filename or get_default_config_file()
        with open(config_filename) as fd:
            cloud_config = yaml.safe_load(fd.read())

    jsonschema.validate(cloud_config, CONFIG_SCHEMA)

    cloud_management_conf = cloud_config['cloud_management']
    cloud_management = _init_driver(cloud_management_conf)

    services = cloud_config.get('services')
    if services:
        cloud_management.update_services(services)
    cloud_management.validate_services()

    containers = cloud_config.get('containers')
    if containers:
        cloud_management.update_containers(containers)
    cloud_management.validate_containers()

    node_discover_conf = cloud_config.get('node_discover')
    if node_discover_conf:
        node_discover = _init_driver(node_discover_conf)
        cloud_management.set_node_discover(node_discover)

    power_managements_conf = cloud_config.get('power_managements')
    if power_managements_conf:
        for pm_conf in power_managements_conf:
            pm = _init_driver(pm_conf)
            cloud_management.add_power_management(pm)

    return cloud_management


def discover(cloud_config):
    """Connect to the cloud and discover nodes and services

    :param cloud_config: dict with cloud and power management params
    :returns: config dict with discovered nodes/services
    """

    cloud_config = copy.deepcopy(cloud_config)
    cloud_management = connect(cloud_config)

    # discover nodes
    hosts = []
    for host in cloud_management.get_nodes().hosts:
        hosts.append({'ip': host.ip, 'mac': host.mac, 'fqdn': host.fqdn})
        LOG.info('Found node: %s' % str(host))
    cloud_config['node_discover'] = {'driver': 'node_list', 'args': hosts}

    # discover services
    cloud_config['services'] = {}
    for service_name in cloud_management.list_supported_services():
        service = cloud_management.get_service(service_name)
        ips = service.get_nodes().get_ips()
        cloud_config['services'][service_name] = {
            'driver': service.NAME,
            'args': service.config
        }
        if ips:
            cloud_config['services'][service_name]['hosts'] = ips
            LOG.info('Found service "%s" on hosts: %s' % (
                service_name, str(ips)))
        else:
            LOG.warning('Service "%s" is not found' % service_name)

    return cloud_config


def human_api(cloud_management, command):
    """Executes a command written as English sentence

    :param cloud_management: library instance as returned by :connect:
           function
    :param command: text command
    """
    human.execute(cloud_management, command)


def register_ansible_modules(paths):
    """Registers ansible modules by provided paths

    Allows to use custom ansible modules in NodeCollection.run_task method

    :param paths: list of paths to folders with ansible modules
    """
    executor.add_module_paths(paths)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

import os_faults


def main():
    # cloud config schema is an extension to os-client-config
    cloud_config = {
        'cloud_management': {
            'driver': 'fuel',
            'args': {
                'address': 'fuel.local',
                'username': 'root',
                'private_key_file': '~/.ssh/os_faults',
            }
        },
        'power_managements': [
            {
                'driver': 'libvirt',
                'args': {
                    'connection_uri': 'qemu+ssh://ubuntu@host.local/system'
                }
            }
        ]
    }

    logging.info('# Create connection to the cloud')
    cloud_management = os_faults.connect(cloud_config)

    logging.info('# Verify connection to the cloud')
    cloud_management.verify()

    # os_faults library operate with 2 types of objects:
    # service - is software that runs in the cloud, e.g. keystone, mysql,
    #           rabbitmq, nova-api, glance-api
    # nodes   - nodes that host the cloud, e.g. hardware server with hostname

    logging.info('# Get nodes where Keystone service is running')
    service = cloud_management.get_service(name='keystone')
    nodes = service.get_nodes()
    logging.info('Nodes: %s', nodes)

    logging.info('# Restart Keystone service on all nodes')
    service.restart()

    logging.info('# Pick and reset one of Keystone service nodes')
    one = nodes.pick()
    one.reset()

    logging.info('# Get all nodes in the cloud')
    nodes = cloud_management.get_nodes()
    logging.info('All cloud nodes: %s', nodes)

    logging.info('# Reset all these nodes')
    nodes.reset()

    logging.info('# Get node by FQDN: node-2.domain.tld')
    nodes = cloud_management.get_nodes(fqdns=['node-2.domain.tld'])
    logging.info('Node node-2.domain.tld: %s', nodes)

    logging.info('# Disable public network on node-2.domain.tld')
    nodes.disconnect(network_name='public')

    logging.info('# Enable public network on node-2.domain.tld')
    nodes.connect(network_name='public')

    logging.info('# Kill Glance API service on a single node')
    service = cloud_management.get_service(name='glance-api')
    nodes = service.get_nodes().pick()
    service.kill(nodes)


if __name__ == '__main__':
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.INFO)
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from docutils import frontend
from docutils import nodes
from docutils.parsers import rst
from docutils import utils


def parse_text(text):
    parser = rst.Parser()
    settings = frontend.OptionParser(
        components=(rst.Parser,)).get_default_values()
    document = utils.new_document(text, settings)
    parser.parse(text, document)
    return document.children


paragraph = lambda text: parse_text(text)[0]
note = lambda msg: nodes.note("", paragraph(msg))
hint = lambda msg: nodes.hint("", *parse_text(msg))
warning = lambda msg: nodes.warning("", paragraph(msg))
category = lambda title: parse_text("%s\n%s" % (title, "-" * len(title)))[0]
subcategory = lambda title: parse_text("%s\n%s" % (title, "~" * len(title)))[0]
section = lambda title: parse_text("%s\n%s" % (title, "\"" * len(title)))[0]


def rstlist(items):
    return parse_text("".join("* %s\n" % item for item in items))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from oslotest import base

import os_faults

LOG = logging.getLogger(__name__)


class TestOSFaultsUniversalDriverLibrary(base.BaseTestCase):
    def test_connection_stack_user(self):
        cloud_config = {
            'cloud_management': {
                'driver': 'universal'
            },
            'node_discover': {
                'driver': 'node_list',
                'args': [
                    {
                        'ip': 'localhost',
                        'auth': {
                            'username': 'stack',
                            'private_key_file': '/opt/stack/.ssh/os-faults-key'
                        }
                    }
                ]
            }
        }

        LOG.info('# Create connection to the cloud')
        cloud_management = os_faults.connect(cloud_config)
        self.assertIsNotNone(cloud_management)

        LOG.info('# Verify connection to the cloud')
        cloud_management.verify()


class TestOSFaultsDevstackDriverLibrary(base.BaseTestCase):
    def test_connection_stack_user(self):
        address = 'localhost'
        cloud_config = {
            'cloud_management': {
                'driver': 'devstack',
                'args':
                    {
                        'address': address,
                        'iface': 'lo',
                        'auth': {
                            'username': 'stack',
                            'private_key_file': '/opt/stack/.ssh/os-faults-key'
                        }
                    }
            }
        }

        LOG.info('# Create connection to the cloud')
        cloud_management = os_faults.connect(cloud_config)
        self.assertIsNotNone(cloud_management)

        LOG.info('# Verify connection to the cloud')
        cloud_management.verify()

        nodes = cloud_management.get_nodes()
        self.assertEqual(1, len(nodes))
        self.assertEqual(address, nodes[0].ip)

        service = cloud_management.get_service('etcd')
        self.assertIsNotNone(service)
        nodes = service.get_nodes()
        self.assertEqual(1, len(nodes))
        self.assertEqual(address, nodes[0].ip)

        selection = nodes.pick()
        self.assertIsNotNone(selection)
        self.assertEqual(1, len(selection))
        self.assertEqual(address, selection[0].ip)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os_faults


def main():
    # The cloud config details could be defined within the script or a
    # separate os-faults.yml file and then loaded to the script.
    # Ex. cloud_management = os_faults.connect(config_filename='os-faults.yml')
    cloud_config = {
        'cloud_management': {
            'driver': 'universal'
        },
        'node_discover': {
            'driver': 'node_list',
            'args': [
                {
                    'ip': '192.0.10.6',
                    'auth': {
                        'username': 'heat-admin',
                        'private_key_file': '/home/stack/.ssh/id_rsa',
                        'become': True
                    }
                },
                {
                    'ip': '192.0.10.8',
                    'auth': {
                        'username': 'heat-admin',
                        'private_key_file': '/home/stack/.ssh/id_rsa',
                        'become': True
                    }
                },
                {
                    'ip': '192.0.10.7',
                    'auth': {
                        'username': 'heat-admin',
                        'private_key_file': '/home/stack/.ssh/id_rsa',
                        'become': True
                    }
                }
            ]
        },
        'services': {
            'openvswitch': {
                'driver': 'system_service',
                'args': {
                    'service_name': 'openvswitch',
                    'grep': 'openvswitch'
                }
            }
        },
        'containers': {
            'neutron_ovs_agent': {
                'driver': 'docker_container',
                'args': {
                    'container_name': 'neutron_ovs_agent'
                }
            },
            'neutron_api': {
                'driver': 'docker_container',
                'args': {
                    'container_name': 'neutron_api'
                }
            }
        }
    }

    logging.info('# Create connection to the cloud')
    cloud_management = os_faults.connect(cloud_config)
    logging.info('Verify connection to the cloud')
    cloud_management.verify()

    logging.info('Get nodes where openvswitch service is running')
    service = cloud_management.get_service(name='openvswitch')
    service_nodes = service.get_nodes()
    logging.info('Nodes: {}'.format(service_nodes))

    logging.info('Stop openvswitch service on random node')
    random_node = service.get_nodes().pick()
    service.terminate(random_node)

    logging.info('Get nodes where neutron_ovs_agent container is running')
    container = cloud_management.get_container(name='neutron_ovs_agent')
    container_nodes = container.get_nodes()
    logging.info('Nodes: {}'.format(container_nodes))

    logging.info('Restart neutron_ovs_agent container on the '
                 'following nodes: {}'.format(container_nodes))
    container.restart(container_nodes)


if __name__ == '__main__':
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.INFO)
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from os_faults.ansible import executor
from os_faults.api import container
from os_faults.api import error

LOG = logging.getLogger(__name__)


class DockerContainers(container.Container):
    """Docker container

    This is docker container driver for any docker containers supported by
    Ansible. Please refer to Ansible documentation
    https://docs.ansible.com/ansible/latest/modules/docker_container_module.html
    for the whole list.

    **Example configuration:**

    .. code-block:: yaml

        containers:
          app:
            driver: docker_container
            args:
              container_name: app

    parameters:

    - **container_name** - name of the container
    """
    NAME = 'docker_container'
    DESCRIPTION = 'Docker container'
    CONFIG_SCHEMA = {
        'type': 'object',
        'properties': {
            'container_name': {'type': 'string'},
        },
        'required': ['container_name'],
        'additionalProperties': False,
    }

    def __init__(self, *args, **kwargs):
        super(DockerContainers, self).__init__(*args, **kwargs)
        self.container_name = self.config['container_name']

    def _run_task(self, nodes, task, message):
        nodes = nodes if nodes is not None else self.get_nodes()
        if len(nodes) == 0:
            raise error.ContainerError(
                'Container %s is not found on any nodes' % self.container_name)

        LOG.info('%s container %s on nodes: %s',
                 message, self.container_name, nodes.get_ips())

        return self.cloud_management.execute_on_cloud(nodes.hosts, task)

    def discover_nodes(self):
        nodes = self.cloud_management.get_nodes()
        cmd = 'bash -c "docker ps | grep \'{}\'"'.format(self.container_name)
        results = self.cloud_management.execute_on_cloud(
            nodes.hosts, {'command': cmd}, False)
        success_ips = [r.host for r in results
                       if r.status == executor.STATUS_OK]
        hosts = [h for h in nodes.hosts if h.ip in success_ips]
        LOG.debug('Container %s is discovered on nodes %s',
                  self.container_name, hosts)
        return self.node_cls(cloud_management=self.cloud_management,
                             hosts=hosts)

    def start(self, nodes=None):
        task = {
            'docker_container': {
                'name': self.container_name, 'state': 'started'
            },
        }
        self._run_task(nodes, task, 'Start')

    def terminate(self, nodes=None):
        task = {
            'docker_container': {
                'name': self.container_name, 'state': 'stopped',
            },
        }
        self._run_task(nodes, task, 'Terminate')

    def restart(self, nodes=None):
        task = {
            'docker_container': {
                'name': self.container_name, 'state': 'started',
                'restart': 'yes'
            },
        }
        self._run_task(nodes, task, 'Restart')



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import collections
import copy
import json
import logging
import os
import shlex
import tempfile

from oslo_concurrency import processutils
import yaml

from os_faults.api import error

LOG = logging.getLogger(__name__)

STDOUT_LIMIT = 4096  # Symbols count
ANSIBLE_FORKS = 100

STATUS_OK = 'OK'
STATUS_FAILED = 'FAILED'
STATUS_UNREACHABLE = 'UNREACHABLE'
STATUS_SKIPPED = 'SKIPPED'

DEFAULT_ERROR_STATUSES = {STATUS_FAILED, STATUS_UNREACHABLE}

SSH_COMMON_ARGS = ('-o UserKnownHostsFile=/dev/null '
                   '-o StrictHostKeyChecking=no '
                   '-o ConnectTimeout=60')


class AnsibleExecutionException(Exception):
    pass


class AnsibleExecutionUnreachable(AnsibleExecutionException):
    pass


AnsibleExecutionRecord = collections.namedtuple(
    'AnsibleExecutionRecord', ['host', 'status', 'task', 'payload'])


def find_ansible():
    stdout, stderr = processutils.execute(
        *shlex.split('which ansible-playbook'), check_exit_code=[0, 1])
    if not stdout:
        raise AnsibleExecutionException(
            'Ansible executable is not found in $PATH')
    return stdout[:-1]


def resolve_relative_path(file_name):
    path = os.path.normpath(os.path.join(
        os.path.dirname(__import__('os_faults').__file__), '../', file_name))
    if os.path.exists(path):
        return path


MODULE_PATHS = {
    resolve_relative_path('os_faults/ansible/modules'),
}


def get_module_paths():
    global MODULE_PATHS
    return MODULE_PATHS


def add_module_paths(paths):
    global MODULE_PATHS
    for path in paths:
        if not os.path.exists(path):
            raise error.OSFError('{!r} does not exist'.format(path))
        # find all subfolders
        dirs = [x[0] for x in os.walk(path)]
        MODULE_PATHS.update(dirs)


def make_module_path_option():
    # now it is a list of strings (MUST have > 1 element)
    module_path = list(get_module_paths())
    if len(module_path) == 1:
        module_path += [module_path[0]]
    return module_path


Options = collections.namedtuple(
    'Options',
    ['connection', 'module_path', 'forks'])


class AnsibleRunner(object):
    def __init__(self, auth=None, forks=ANSIBLE_FORKS, serial=None):
        super(AnsibleRunner, self).__init__()

        self.default_host_vars = self._build_auth_host_vars(auth)
        self.options = Options(
            connection='smart',
            module_path=make_module_path_option(),
            forks=forks)
        self.serial = serial or 10
        self.ansible = find_ansible()

    @staticmethod
    def _build_proxy_arg(jump_user, jump_host, private_key_file=None):
        key = '-i ' + private_key_file if private_key_file else ''
        return (' -o ProxyCommand="ssh %(key)s -W %%h:%%p %(ssh_args)s '
                '%(user)s@%(host)s"'
                % dict(key=key, user=jump_user,
                       host=jump_host, ssh_args=SSH_COMMON_ARGS))

    def _run_play(self, play_source, host_vars):
        inventory = {}

        for host, variables in host_vars.items():
            host_vars = dict((k, v)
                             for k, v in self.default_host_vars.items() if v)
            host_vars.update(dict((k, v) for k, v in variables.items() if v))

            inventory[host] = host_vars
            inventory[host]['ansible_connection'] = self.options.connection

        full_inventory = {'all': {'hosts': inventory}}

        temp_dir = tempfile.mkdtemp(prefix='os-faults')
        inventory_file_name = os.path.join(temp_dir, 'inventory')
        playbook_file_name = os.path.join(temp_dir, 'playbook')

        with open(inventory_file_name, 'w') as fd:
            cnt = yaml.safe_dump(full_inventory, default_flow_style=False)
            print(cnt, file=fd)
            LOG.debug('Inventory:\n%s', cnt)

        play = {
            'hosts': 'all',
            'gather_facts': 'no',
            'tasks': play_source['tasks'],
        }

        with open(playbook_file_name, 'w') as fd:
            cnt = yaml.safe_dump([play], default_flow_style=False)
            print(cnt, file=fd)
            LOG.debug('Playbook:\n%s', cnt)

        cmd = ('%(ansible)s --module-path %(module_path)s '
               '-i %(inventory)s %(playbook)s' %
               {'ansible': self.ansible,
                'module_path': ':'.join(self.options.module_path),
                'inventory': inventory_file_name,
                'playbook': playbook_file_name})

        logging.info('Executing %s' % cmd)
        command_stdout, command_stderr = processutils.execute(
            *shlex.split(cmd),
            env_variables={'ANSIBLE_STDOUT_CALLBACK': 'json'},
            check_exit_code=False)

        d = json.loads(command_stdout[command_stdout.find('{'):])
        h = d['plays'][0]['tasks'][0]['hosts']
        recs = []
        for h, hv in h.items():
            if hv.get('unreachable'):
                status = STATUS_UNREACHABLE
            elif hv.get('failed'):
                status = STATUS_FAILED
            else:
                status = STATUS_OK
            r = AnsibleExecutionRecord(host=h, status=status, task='',
                                       payload=hv)
            recs.append(r)

        return recs

    def run_playbook(self, playbook, host_vars):
        result = []

        for play_source in playbook:
            play_source['gather_facts'] = 'no'

            result += self._run_play(play_source, host_vars)

        return result

    def execute(self, hosts, task, raise_on_statuses=None):
        """Executes the task on every host from the list

        Raises exception if any of the commands fails with one of specified
        statuses.
        :param hosts: list of host addresses
        :param task: Ansible task
        :param raise_on_statuses: raise exception if any of commands return
        any of these statuses
        :return: execution result, type AnsibleExecutionRecord
        """
        if raise_on_statuses is None:
            raise_on_statuses = DEFAULT_ERROR_STATUSES

        LOG.debug('Executing task: %s on hosts: %s with serial: %s',
                  task, hosts, self.serial)

        host_vars = {h.ip: self._build_auth_host_vars(h.auth) for h in hosts}
        task_play = {'hosts': [h.ip for h in hosts],
                     'tasks': [task],
                     'serial': self.serial}
        result = self.run_playbook([task_play], host_vars)

        log_result = copy.deepcopy(result)
        LOG.debug('Execution completed with %s result(s):' % len(log_result))
        for lr in log_result:
            if 'stdout' in lr.payload:
                if len(lr.payload['stdout']) > STDOUT_LIMIT:
                    lr.payload['stdout'] = (
                        lr.payload['stdout'][:STDOUT_LIMIT] + '... <cut>')
            if 'stdout_lines' in lr.payload:
                del lr.payload['stdout_lines']
            LOG.debug(lr)

        if raise_on_statuses:
            errors = []
            only_unreachable = True

            for r in result:
                if r.status in raise_on_statuses:
                    if r.status != STATUS_UNREACHABLE:
                        only_unreachable = False
                    errors.append(r)

            if errors:
                msg = 'Execution failed: %s' % ', '.join((
                    '(host: %s, status: %s)' % (r.host, r.status))
                    for r in errors)
                ek = (AnsibleExecutionUnreachable if only_unreachable
                      else AnsibleExecutionException)
                raise ek(msg)

        return result

    def _build_auth_host_vars(self, auth):
        if not auth:
            return {}

        ssh_common_args = SSH_COMMON_ARGS
        if 'jump' in auth:
            ssh_common_args += self._build_proxy_arg(
                jump_host=auth['jump']['host'],
                jump_user=auth['jump'].get(
                    'username', auth.get('username')),
                private_key_file=auth['jump'].get('private_key_file'))

        return {
            'ansible_user': auth.get('username'),
            'ansible_ssh_pass': auth.get('password'),
            'ansible_become_user': auth.get('become_username'),
            'ansible_become_pass': auth.get('become_password'),
            'ansible_become_method': auth.get('become_method'),
            'ansible_ssh_private_key_file': auth.get('private_key_file'),
            'ansible_ssh_common_args': ssh_common_args,
        }



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from os_faults.ansible import executor
from os_faults.api import cloud_management
from os_faults.api import error

LOG = logging.getLogger(__name__)


class UniversalCloudManagement(cloud_management.CloudManagement):
    """Universal cloud management driver

    This driver is suitable for the most abstract (and thus universal) case.
    The driver does not have any built-in services nor node discovery
    capabilities. All services need to be listed explicitly in a config file.
    Node list is specified using `node_list` node discovery driver.

    **Example of multi-node configuration:**

    Note that in this configuration a node discovery driver is required.

    .. code-block:: yaml

        cloud_management:
          driver: universal

        node_discover:
          driver: node_list
          args:
            - ip: 192.168.5.149
              auth:
                username: developer
                private_key_file: cloud_key
                become_password: my_secret_password
            - ip: 192.168.5.150
              auth:
                username: developer
                private_key_file: cloud_key
                become_password: my_secret_password

    """

    NAME = 'universal'
    DESCRIPTION = 'Universal cloud management driver'
    CONFIG_SCHEMA = {
        'type': 'object',
        '$schema': 'http://json-schema.org/draft-04/schema#',
        'properties': {},
        'additionalProperties': False,
    }

    def __init__(self, cloud_management_params):
        super(UniversalCloudManagement, self).__init__()

        self.cloud_executor = executor.AnsibleRunner()

    def verify(self):
        """Verify connection to the cloud."""
        nodes = self.get_nodes()
        if not nodes:
            raise error.OSFError('Cloud has no nodes')

        task = {'command': 'hostname'}
        task_result = self.execute_on_cloud(nodes.hosts, task)
        LOG.debug('Host names of cloud nodes: %s',
                  ', '.join(r.payload['stdout'] for r in task_result))

        LOG.info('Connected to cloud successfully!')

    def execute_on_cloud(self, hosts, task, raise_on_error=True):
        """Execute task on specified hosts within the cloud.

        :param hosts: List of host FQDNs
        :param task: Ansible task
        :param raise_on_error: throw exception in case of error
        :return: Ansible execution result (list of records)
        """
        if raise_on_error:
            return self.cloud_executor.execute(hosts, task)
        else:
            return self.cloud_executor.execute(hosts, task, [])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import inspect
import logging
import re

from os_faults.api import container as container_pkg
from os_faults.api import error
from os_faults.api import node_collection as node_collection_pkg
from os_faults.api import service as service_pkg
from os_faults.api import utils

LOG = logging.getLogger(__name__)

"""
Human API understands commands like these (examples):
 * restart <service> service [on (random|one|single|<fqdn> node[s])]
 * terminate <service> service [on (random|one|single|<fqdn> node[s])]
 * start <service> service [on (random|one|single|<fqdn> node[s])]
 * kill <service> service [on (random|one|single|<fqdn> node[s])]
 * plug <service> service [on (random|one|single|<fqdn> node[s])]
 * unplug <service> service [on (random|one|single|<fqdn> node[s])]
 * freeze <service> service [on (random|one|single|<fqdn> node[s])]
   [for <T> seconds]
 * unfreeze <service> service [on (random|one|single|<fqdn> node[s])]
 * reboot [random|one|single|<fqdn>] node[s] [with <service> service]
 * reset [random|one|single|<fqdn>] node[s] [with <service> service]
 * stress [cpu|memory|disk|kernel for <T> seconds] on
   [random|one|single|<fqdn>] node[s] [with <service> service]
 * disconnect <name> network on [random|one|single|<fqdn>] node[s]
   [with <service> service]
 * connect <name> network on [random|one|single|<fqdn>] node[s]
   [with <service> service]
"""


def list_actions(klazz):
    return set(m[0].replace('_', ' ') for m in inspect.getmembers(
        klazz,
        predicate=utils.is_public))

RANDOMNESS = {'one', 'random', 'some', 'single'}
ANYTHING = {'all'}
NODE_ALIASES_PATTERN = '|'.join(RANDOMNESS | ANYTHING)
SERVICE_ACTIONS = list_actions(service_pkg.Service)
SERVICE_ACTIONS_PATTERN = '|'.join(SERVICE_ACTIONS)
CONTAINER_ACTIONS = list_actions(container_pkg.Container)
CONTAINER_ACTIONS_PATTERN = '|'.join(CONTAINER_ACTIONS)
NODE_ACTIONS = list_actions(node_collection_pkg.NodeCollection)
NODE_ACTIONS_PATTERN = '|'.join(NODE_ACTIONS)

PATTERNS = [
    re.compile(r'(?P<action>%s)'
               r'\s+(?P<service>\S+)\s+service'
               r'(\s+on(\s+(?P<node>\S+))?\s+nodes?)?'
               r'(\s+for\s+(?P<duration>\d+)\s+seconds)?' %
               SERVICE_ACTIONS_PATTERN),
    re.compile(r'(?P<action>%s)'
               r'\s+(?P<container>\S+)\s+container'
               r'(\s+on(\s+(?P<node>\S+))?\s+nodes?)?'
               r'(\s+for\s+(?P<duration>\d+)\s+seconds)?' %
               CONTAINER_ACTIONS_PATTERN),
    re.compile(r'(?P<action>%s)'
               r'(\s+(?P<network>\w+)\s+network\s+on)?'
               r'(\s+(?P<target>\w+)'
               r'(\s+for\s+(?P<duration>\d+)\s+seconds)(\s+on)?)?'
               r'(\s+(?P<node>%s|\S+))?'
               r'\s+nodes?'
               r'(\s+with\s+(?P<service>\S+)\s+service)?' %
               (NODE_ACTIONS_PATTERN, NODE_ALIASES_PATTERN)),
]


def execute(destructor, command):
    command = command.lower()
    rec = None
    for pattern in PATTERNS:
        rec = re.search(pattern, command)
        if rec:
            break

    if not rec:
        raise error.OSFException('Could not parse command: %s' % command)

    groups = rec.groupdict()

    action = groups.get('action').replace(' ', '_')
    service_name = groups.get('service')
    container_name = groups.get('container')
    node_name = groups.get('node')
    network_name = groups.get('network')
    target = groups.get('target')
    duration = groups.get('duration')

    if service_name:
        service = destructor.get_service(name=service_name)

        if action in SERVICE_ACTIONS:

            kwargs = {}
            if node_name in RANDOMNESS:
                kwargs['nodes'] = service.get_nodes().pick()
            elif node_name and node_name not in ANYTHING:
                kwargs['nodes'] = destructor.get_nodes(fqdns=[node_name])

            if duration:
                kwargs['sec'] = int(duration)

            fn = getattr(service, action)
            fn(**kwargs)

        else:  # node actions
            nodes = service.get_nodes()

            if node_name in RANDOMNESS:
                nodes = nodes.pick()

            kwargs = {}
            if network_name:
                kwargs['network_name'] = network_name
            if target:
                kwargs['target'] = target
                kwargs['duration'] = int(duration)

            fn = getattr(nodes, action)
            fn(**kwargs)
    elif container_name:
        container = destructor.get_container(name=container_name)

        if action in CONTAINER_ACTIONS:

            kwargs = {}
            if node_name in RANDOMNESS:
                kwargs['nodes'] = container.get_nodes().pick()
            elif node_name and node_name not in ANYTHING:
                kwargs['nodes'] = destructor.get_nodes(fqdns=[node_name])

            if duration:
                kwargs['sec'] = int(duration)

            fn = getattr(container, action)
            fn(**kwargs)

        else:  # node actions
            nodes = container.get_nodes()

            if node_name in RANDOMNESS:
                nodes = nodes.pick()

            kwargs = {}
            if network_name:
                kwargs['network_name'] = network_name
            if target:
                kwargs['target'] = target
                kwargs['duration'] = int(duration)

            fn = getattr(nodes, action)
            fn(**kwargs)
    else:  # nodes operation
        if node_name and node_name not in ANYTHING:
            nodes = destructor.get_nodes(fqdns=[node_name])
        else:
            nodes = destructor.get_nodes()

        kwargs = {}
        if network_name:
            kwargs['network_name'] = network_name
        if target:
            kwargs['target'] = target
            kwargs['duration'] = int(duration)

        fn = getattr(nodes, action)
        fn(**kwargs)

    LOG.info('Command `%s` is executed successfully', command)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

import six

from os_faults.api import base_driver
from os_faults.api.utils import public


@six.add_metaclass(abc.ABCMeta)
class Container(base_driver.BaseDriver):

    def __init__(self, container_name, config, node_cls, cloud_management,
                 hosts=None):
        self.container_name = container_name
        self.config = config
        self.node_cls = node_cls
        self.cloud_management = cloud_management
        self.hosts = hosts

    @abc.abstractmethod
    def discover_nodes(self):
        """Discover nodes where this Container is running

        :returns: NodesCollection
        """

    def get_nodes(self):
        """Get nodes where this Container is running

        :returns: NodesCollection
        """
        if self.hosts is not None:
            nodes = self.cloud_management.get_nodes()
            hosts = [h for h in nodes.hosts if h.ip in self.hosts]
            return self.node_cls(cloud_management=self.cloud_management,
                                 hosts=hosts)
        return self.discover_nodes()

    @public
    def start(self, nodes=None):
        """Start Container on all nodes or on particular subset

        :param nodes: NodesCollection
        """
        raise NotImplementedError

    @public
    def terminate(self, nodes=None):
        """Terminate Container gracefully on all nodes or on particular subset

        :param nodes: NodesCollection
        """
        raise NotImplementedError

    @public
    def restart(self, nodes=None):
        """Restart Container on all nodes or on particular subset

        :param nodes: NodesCollection
        """
        raise NotImplementedError



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import ddt
import mock

from os_faults.api import node_collection
from os_faults.drivers.cloud import devstack
from os_faults.tests.unit import fakes
from os_faults.tests.unit import test


class DevStackNodeTestCase(test.TestCase):

    def setUp(self):
        super(DevStackNodeTestCase, self).setUp()
        self.mock_cloud_management = mock.Mock(
            spec=devstack.DevStackCloudManagement)
        self.host = node_collection.Host(
            ip='10.0.0.2', mac='09:7b:74:90:63:c1', fqdn='')

        self.node_collection = devstack.DevStackNodeCollection(
            cloud_management=self.mock_cloud_management,
            hosts=[copy.deepcopy(self.host)])

    def test_connect(self):
        pass

    def test_disconnect(self):
        pass


@ddt.ddt
class DevStackManagementTestCase(test.TestCase):

    def setUp(self):
        super(DevStackManagementTestCase, self).setUp()
        self.conf = {'address': '10.0.0.2', 'auth': {'username': 'root'}}
        self.host = node_collection.Host('10.0.0.2')
        self.discoverd_host = node_collection.Host(ip='10.0.0.2',
                                                   mac='09:7b:74:90:63:c1',
                                                   fqdn='')

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    def test_verify(self, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''},
                                     host='10.0.0.2')],
        ]
        devstack_management = devstack.DevStackCloudManagement(self.conf)
        devstack_management.verify()

        ansible_runner_inst.execute.assert_has_calls([
            mock.call([self.host],
                      {'command': 'cat /sys/class/net/eth0/address'}),
        ])

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    def test_execute_on_cloud(self, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '/root'})],
        ]
        devstack_management = devstack.DevStackCloudManagement(self.conf)
        result = devstack_management.execute_on_cloud(
            ['10.0.0.2'], {'command': 'pwd'})

        ansible_runner_inst.execute.assert_called_once_with(
            ['10.0.0.2'], {'command': 'pwd'})
        self.assertEqual(
            [fakes.FakeAnsibleResult(payload={'stdout': '/root'})], result)

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    def test_get_nodes(self, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
        ]

        devstack_management = devstack.DevStackCloudManagement(self.conf)
        nodes = devstack_management.get_nodes()
        ansible_runner_inst.execute.assert_called_once_with(
            [self.host], {'command': 'cat /sys/class/net/eth0/address'})

        self.assertIsInstance(nodes, devstack.DevStackNodeCollection)
        self.assertEqual(
            [node_collection.Host(ip='10.0.0.2', mac='09:7b:74:90:63:c1',
                                  fqdn='')],
            nodes.hosts)

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    @ddt.data(*devstack.DevStackCloudManagement.SERVICES.keys())
    def test_get_service_nodes(self, service_name, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')]
        ]

        devstack_management = devstack.DevStackCloudManagement(self.conf)

        service = devstack_management.get_service(service_name)
        nodes = service.get_nodes()

        cmd = 'bash -c "ps ax | grep -v grep | grep \'{}\'"'.format(
            service.grep)
        ansible_runner_inst.execute.assert_has_calls([
            mock.call(
                [self.host], {'command': 'cat /sys/class/net/eth0/address'}),
            mock.call([self.discoverd_host], {'command': cmd}, [])
        ])
        self.assertEqual(
            [node_collection.Host(ip='10.0.0.2', mac='09:7b:74:90:63:c1',
                                  fqdn='')],
            nodes.hosts)

    @mock.patch('os_faults.ansible.executor.AnsibleRunner')
    def test_validate_services(self, _):
        devstack_management = devstack.DevStackCloudManagement(self.conf)
        devstack_management.validate_services()


@ddt.ddt
class DevStackServiceTestCase(test.TestCase):

    def setUp(self):
        super(DevStackServiceTestCase, self).setUp()
        self.conf = {'address': '10.0.0.2', 'auth': {'username': 'stack'}}
        self.host = node_collection.Host('10.0.0.2')
        self.discoverd_host = node_collection.Host(ip='10.0.0.2',
                                                   mac='09:7b:74:90:63:c1',
                                                   fqdn='')

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    @ddt.data(*devstack.DevStackCloudManagement.SERVICES.keys())
    def test_restart(self, service_name, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')]
        ]

        devstack_management = devstack.DevStackCloudManagement(
            self.conf)

        service = devstack_management.get_service(service_name)
        service.restart()

        cmd = 'bash -c "ps ax | grep -v grep | grep \'{}\'"'.format(
            service.grep)
        ansible_runner_inst.execute.assert_has_calls([
            mock.call(
                [self.host], {'command': 'cat /sys/class/net/eth0/address'}),
            mock.call([self.discoverd_host], {'command': cmd}, []),
        ])

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    @ddt.data(*devstack.DevStackCloudManagement.SERVICES.keys())
    def test_terminate(self, service_name, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')]
        ]

        devstack_management = devstack.DevStackCloudManagement(
            self.conf)

        service = devstack_management.get_service(service_name)
        service.terminate()

        cmd = 'bash -c "ps ax | grep -v grep | grep \'{}\'"'.format(
            service.grep)
        ansible_runner_inst.execute.assert_has_calls([
            mock.call(
                [self.host], {'command': 'cat /sys/class/net/eth0/address'}),
            mock.call([self.discoverd_host], {'command': cmd}, []),
        ])

    @mock.patch('os_faults.ansible.executor.AnsibleRunner', autospec=True)
    @ddt.data(*devstack.DevStackCloudManagement.SERVICES.keys())
    def test_start(self, service_name, mock_ansible_runner):
        ansible_runner_inst = mock_ansible_runner.return_value
        ansible_runner_inst.execute.side_effect = [
            [fakes.FakeAnsibleResult(payload={'stdout': '09:7b:74:90:63:c1'},
                                     host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')],
            [fakes.FakeAnsibleResult(payload={'stdout': ''}, host='10.0.0.2')]
        ]

        devstack_management = devstack.DevStackCloudManagement(
            self.conf)

        service = devstack_management.get_service(service_name)
        service.start()

        cmd = 'bash -c "ps ax | grep -v grep | grep \'{}\'"'.format(
            service.grep)
        ansible_runner_inst.execute.assert_has_calls([
            mock.call(
                [self.host], {'command': 'cat /sys/class/net/eth0/address'}),
            mock.call([self.discoverd_host], {'command': cmd}, []),
        ])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import signal

from os_faults.ansible import executor
from os_faults.api import error
from os_faults.api import service
from os_faults import utils

LOG = logging.getLogger(__name__)

PORT_SCHEMA = {
    'type': 'array',
    'items': [
        {'enum': ['tcp', 'udp']},
        {'type': 'integer', 'minimum': 0, 'maximum': 65535},
    ],
    'minItems': 2,
    'maxItems': 2,
}


class ServiceAsProcess(service.Service):
    """Service as process

    "process" is a basic service driver that uses `ps` and `kill` in
    actions like kill / freeze / unfreeze. Commands for start / restart
    / terminate should be specified in configuration, otherwise
    the commands will fail at runtime.

    **Example configuration:**

    .. code-block:: yaml

        services:
          app:
            driver: process
            args:
              grep: my_app
              restart_cmd: /bin/my_app --restart
              terminate_cmd: /bin/stop_my_app
              start_cmd: /bin/my_app
              port: ['tcp', 4242]

    parameters:

    - **grep** - regexp for grep to find process PID
    - **restart_cmd** - command to restart service (optional)
    - **terminate_cmd** - command to terminate service (optional)
    - **start_cmd** - command to start service (optional)
    - **port** - tuple with two values - protocol, port number (optional)

    """

    NAME = 'process'
    DESCRIPTION = 'Service as process'
    CONFIG_SCHEMA = {
        'type': 'object',
        'properties': {
            'grep': {'type': 'string'},
            'start_cmd': {'type': 'string'},
            'terminate_cmd': {'type': 'string'},
            'restart_cmd': {'type': 'string'},
            'port': PORT_SCHEMA,
        },
        'required': ['grep'],
        'additionalProperties': False,
    }

    def __init__(self, *args, **kwargs):
        super(ServiceAsProcess, self).__init__(*args, **kwargs)
        self.grep = self.config['grep']
        self.start_cmd = self.config.get('start_cmd')
        self.terminate_cmd = self.config.get('terminate_cmd')
        self.restart_cmd = self.config.get('restart_cmd')
        self.port = self.config.get('port')

    def _run_task(self, nodes, task, message):
        nodes = nodes if nodes is not None else self.get_nodes()
        if len(nodes) == 0:
            raise error.ServiceError(
                'Service %s is not found on any nodes' % self.service_name)

        LOG.info('%s service %s on nodes: %s',
                 message, self.service_name, nodes.get_ips())

        return self.cloud_management.execute_on_cloud(nodes.hosts, task)

    def discover_nodes(self):
        nodes = self.cloud_management.get_nodes()
        cmd = 'bash -c "ps ax | grep -v grep | grep \'{}\'"'.format(self.grep)
        results = self.cloud_management.execute_on_cloud(
            nodes.hosts, {'command': cmd}, False)
        success_ips = [r.host for r in results
                       if r.status == executor.STATUS_OK]
        hosts = [h for h in nodes.hosts if h.ip in success_ips]
        LOG.debug('Service %s is discovered on nodes %s',
                  self.service_name, hosts)
        return self.node_cls(cloud_management=self.cloud_management,
                             hosts=hosts)

    @utils.require_variables('restart_cmd')
    def restart(self, nodes=None):
        self._run_task(nodes, {'shell': self.restart_cmd}, 'Restart')

    @utils.require_variables('terminate_cmd')
    def terminate(self, nodes=None):
        self._run_task(nodes, {'shell': self.terminate_cmd}, 'Terminate')

    @utils.require_variables('start_cmd')
    def start(self, nodes=None):
        self._run_task(nodes, {'shell': self.start_cmd}, 'Start')

    def kill(self, nodes=None):
        task = {
            'kill': {
                'grep': self.grep, 'sig': signal.SIGKILL
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, 'Kill')

    def freeze(self, nodes=None, sec=None):
        if sec:
            task = {
                'freeze': {
                    'grep': self.grep, 'sec': sec
                },
                'become': 'yes',
            }
        else:
            task = {
                'kill': {
                    'grep': self.grep, 'sig': signal.SIGSTOP
                },
                'become': 'yes',
            }
        message = "Freeze %s" % (('for %s sec ' % sec) if sec else '')
        self._run_task(nodes, task, message)

    def unfreeze(self, nodes=None):
        task = {
            'kill': {
                'grep': self.grep, 'sig': signal.SIGCONT
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, 'Unfreeze')

    @utils.require_variables('port')
    def plug(self, nodes=None):
        nodes = nodes if nodes is not None else self.get_nodes()
        message = "Open port %d for" % self.port[1]
        task = {
            'iptables': {
                'protocol': self.port[0], 'port': self.port[1],
                'action': 'unblock', 'service': self.service_name
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, message)

    @utils.require_variables('port')
    def unplug(self, nodes=None):
        nodes = nodes if nodes is not None else self.get_nodes()
        message = "Close port %d for" % self.port[1]
        task = {
            'iptables': {
                'protocol': self.port[0], 'port': self.port[1],
                'action': 'block', 'service': self.service_name
            },
            'become': 'yes',
        }
        self._run_task(nodes, task, message)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import random

from os_faults.api import error
from os_faults.api.utils import public
from os_faults import utils

LOG = logging.getLogger(__name__)


class Host(utils.ComparableMixin, utils.ReprMixin):

    ATTRS = ('ip', 'mac', 'fqdn', 'libvirt_name')

    def __init__(self, ip, mac=None, fqdn=None, libvirt_name=None, auth=None):
        self.ip = ip
        self.mac = mac
        self.fqdn = fqdn
        self.libvirt_name = libvirt_name
        self.auth = auth


class NodeCollection(utils.ReprMixin):

    ATTRS = ('hosts', )

    def __init__(self, cloud_management=None, hosts=None):
        self.cloud_management = cloud_management
        self._hosts = set(hosts)

    @property
    def hosts(self):
        return sorted(self._hosts)

    def __len__(self):
        return len(self._hosts)

    def _check_nodes_types(self, other):
        if type(self) is not type(other):
            raise TypeError(
                'Unsupported operand types: {} and {}'.format(
                    type(self), type(other)))
        if self.cloud_management is not other.cloud_management:
            raise error.NodeCollectionError(
                'NodeCollections have different cloud_managements: '
                '{} and {}'.format(self.cloud_management,
                                   other.cloud_management))

    def __add__(self, other):
        return self.__or__(other)

    def __sub__(self, other):
        self._check_nodes_types(other)
        return self._make_instance(self._hosts - other._hosts)

    def __and__(self, other):
        self._check_nodes_types(other)
        return self._make_instance(self._hosts & other._hosts)

    def __or__(self, other):
        self._check_nodes_types(other)
        return self._make_instance(self._hosts | other._hosts)

    def __xor__(self, other):
        self._check_nodes_types(other)
        return self._make_instance(self._hosts ^ other._hosts)

    def __contains__(self, host):
        return host in self._hosts

    def __iter__(self):
        for host in self.hosts:
            yield host

    def __getitem__(self, item):
        return self.hosts[item]

    def _make_instance(self, hosts):
        return self.__class__(cloud_management=self.cloud_management,
                              hosts=hosts)

    def get_ips(self):
        return [host.ip for host in self.hosts]

    def get_macs(self):
        return [host.mac for host in self.hosts]

    def get_fqdns(self):
        return [host.fqdn for host in self.hosts]

    def pick(self, count=1):
        """Pick one Node out of collection

        :return: NodeCollection consisting just one node
        """
        if count > len(self._hosts):
            msg = 'Cannot pick {} from {} node(s)'.format(
                count, len(self._hosts))
            raise error.NodeCollectionError(msg)
        return self._make_instance(random.sample(self._hosts, count))

    def filter(self, criteria_fn):
        hosts = list(filter(criteria_fn, self._hosts))
        if hosts:
            return self._make_instance(hosts)
        else:
            raise error.NodeCollectionError(
                'No nodes found according to criterion')

    def run_task(self, task, raise_on_error=True):
        """Run ansible task on node colection

        :param task: ansible task as dict
        :param raise_on_error: throw exception in case of error
        :return: AnsibleExecutionRecord with results of task
        """
        LOG.info('Run task: %s on nodes: %s', task, self)
        return self.cloud_management.execute_on_cloud(
            self.hosts, task, raise_on_error=raise_on_error)

    @public
    def reboot(self):
        """Reboot all nodes gracefully

        """
        LOG.info('Reboot nodes: %s', self)
        task = {'command': 'reboot now'}
        self.cloud_management.execute_on_cloud(self.hosts, task)

    @public
    def oom(self):
        """Fill all node's RAM

        """
        raise NotImplementedError

    @public
    def poweroff(self):
        """Power off all nodes abruptly

        """
        LOG.info('Power off nodes: %s', self)
        self.cloud_management.power_manager.poweroff(self.hosts)

    @public
    def poweron(self):
        """Power on all nodes abruptly

        """
        LOG.info('Power on nodes: %s', self)
        self.cloud_management.power_manager.poweron(self.hosts)

    @public
    def reset(self):
        """Reset (cold restart) all nodes

        """
        LOG.info('Reset nodes: %s', self)
        self.cloud_management.power_manager.reset(self.hosts)

    @public
    def shutdown(self):
        """Shutdown all nodes gracefully

        """
        LOG.info('Shutdown nodes: %s', self)
        self.cloud_management.power_manager.shutdown(self.hosts)

    def snapshot(self, snapshot_name, suspend=True):
        """Create snapshot for all nodes

        """
        LOG.info('Create snapshot "%s" for nodes: %s', snapshot_name, self)
        self.cloud_management.power_manager.snapshot(
            self.hosts, snapshot_name, suspend)

    def revert(self, snapshot_name, resume=True):
        """Revert snapshot for all nodes

        """
        LOG.info('Revert snapshot "%s" for nodes: %s', snapshot_name, self)
        self.cloud_management.power_manager.revert(
            self.hosts, snapshot_name, resume)

    @public
    def disconnect(self, network_name):
        """Disconnect nodes from <network_name> network

        :param network_name: name of network
        """
        raise NotImplementedError

    @public
    def connect(self, network_name):
        """Connect nodes to <network_name> network

        :param network_name: name of network
        """
        raise NotImplementedError

    @public
    def stress(self, target, duration=None):
        """Stress node OS and hardware

        """
        duration = duration or 10  # defaults to 10 seconds
        LOG.info('Stress %s for %ss on nodes %s', target, duration, self)
        task = {'stress': {
            'target': target,
            'duration': duration,
        }}
        self.cloud_management.execute_on_cloud(self.hosts, task)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from os_faults.ansible import executor


class FakeAnsibleResult(object):

    def __init__(self, payload, host=None, status=executor.STATUS_OK):
        self.host = host
        self.payload = payload
        self.status = status

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return NotImplemented
        if self.host != other.host:
            return False
        if self.payload != other.payload:
            return False
        if self.status != other.status:
            return False
        return True

    def __ne__(self, other):
        return not self == other



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from oslo_utils import importutils

from os_faults.api import error
from os_faults.api import power_management

LOG = logging.getLogger(__name__)


class LibvirtDriver(power_management.PowerDriver):
    """Libvirt driver.

    **Example configuration:**

    .. code-block:: yaml

        power_managements:
        - driver: libvirt
          args:
            connection_uri: qemu+unix:///system

    parameters:

    - **connection_uri** - libvirt uri

    """

    NAME = 'libvirt'
    DESCRIPTION = 'Libvirt power management driver'
    CONFIG_SCHEMA = {
        'type': 'object',
        '$schema': 'http://json-schema.org/draft-04/schema#',
        'properties': {
            'connection_uri': {'type': 'string'},
        },
        'required': ['connection_uri'],
        'additionalProperties': False,
    }

    def __init__(self, params):
        self.connection_uri = params['connection_uri']
        self._cached_conn = None

    @property
    def conn(self):
        return self._get_connection()

    def _get_connection(self):
        if self._cached_conn is None:
            libvirt_module = importutils.try_import('libvirt')
            if not libvirt_module:
                raise error.OSFError('libvirt-python is required '
                                     'to use LibvirtDriver')
            self._cached_conn = libvirt_module.open(self.connection_uri)

        return self._cached_conn

    def _find_domain_by_host(self, host):
        for domain in self.conn.listAllDomains():
            if host.libvirt_name and host.libvirt_name == domain.name():
                return domain
            if host.mac and host.mac in domain.XMLDesc():
                return domain

        raise error.PowerManagementError(
            'Domain not found for host %s.' % host)

    def supports(self, host):
        try:
            self._find_domain_by_host(host)
        except error.PowerManagementError:
            return False
        return True

    def poweroff(self, host):
        domain = self._find_domain_by_host(host)
        LOG.debug('Power off domain with name: %s', host.mac)
        domain.destroy()
        LOG.info('Domain powered off: %s', host.mac)

    def poweron(self, host):
        domain = self._find_domain_by_host(host)
        LOG.debug('Power on domain with name: %s', domain.name())
        domain.create()
        LOG.info('Domain powered on: %s', domain.name())

    def reset(self, host):
        domain = self._find_domain_by_host(host)
        LOG.debug('Reset domain with name: %s', domain.name())
        domain.reset()
        LOG.info('Domain reset: %s', domain.name())

    def shutdown(self, host):
        domain = self._find_domain_by_host(host)
        LOG.debug('Shutdown domain with name: %s', domain.name())
        domain.shutdown()
        LOG.info('Domain is off: %s', domain.name())

    def snapshot(self, host, snapshot_name, suspend):
        domain = self._find_domain_by_host(host)
        LOG.debug('Create snapshot "%s" for domain with name: %s',
                  snapshot_name, domain.name())
        if suspend:
            domain.suspend()
        domain.snapshotCreateXML(
            '<domainsnapshot><name>{}</name></domainsnapshot>'.format(
                snapshot_name))
        if suspend:
            domain.resume()
        LOG.debug('Created snapshot "%s" for domain with name: %s',
                  snapshot_name, domain.name())

    def revert(self, host, snapshot_name, resume):
        domain = self._find_domain_by_host(host)
        LOG.debug('Revert snapshot "%s" for domain with name: %s',
                  snapshot_name, domain.name())
        snapshot = domain.snapshotLookupByName(snapshot_name)
        if domain.isActive():
            domain.destroy()
        domain.revertToSnapshot(snapshot)
        if resume:
            domain.resume()
        LOG.debug('Reverted snapshot "%s" for domain with name: %s',
                  snapshot_name, domain.name())
